@@ -59,16 +59,16 @@ U_NAMESPACE_USE
 
 U_CAPI int32_t
 u_formatMessage(const char  *locale,
-                const UChar *pattern,
+                const char16_t *pattern,
                 int32_t     patternLength,
-                UChar       *result,
+                char16_t    *result,
                 int32_t     resultLength,
                 UErrorCode  *status,
                 ...)
 {
     va_list    ap;
     int32_t actLen;        
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     // start vararg processing
     va_start(ap, status);
 
@@ -81,16 +81,16 @@ u_formatMessage(const char  *locale,
 
 U_CAPI int32_t U_EXPORT2
 u_vformatMessage(   const char  *locale,
-                    const UChar *pattern,
+                    const char16_t *pattern,
                     int32_t     patternLength,
-                    UChar       *result,
+                    char16_t    *result,
                     int32_t     resultLength,
                     va_list     ap,
                     UErrorCode  *status)
 
 {
-    //argument checking defered to subsequent method calls
-    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
+    //argument checking deferred to subsequent method calls
+    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,nullptr,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
     return retVal;
@@ -98,9 +98,9 @@ u_vformatMessage(   const char  *locale,
 
 U_CAPI int32_t
 u_formatMessageWithError(const char *locale,
-                        const UChar *pattern,
+                        const char16_t *pattern,
                         int32_t     patternLength,
-                        UChar       *result,
+                        char16_t    *result,
                         int32_t     resultLength,
                         UParseError *parseError,
                         UErrorCode  *status,
@@ -108,7 +108,7 @@ u_formatMessageWithError(const char *locale,
 {
     va_list    ap;
     int32_t actLen;
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     // start vararg processing
     va_start(ap, status);
 
@@ -121,16 +121,16 @@ u_formatMessageWithError(const char *locale,
 
 U_CAPI int32_t U_EXPORT2
 u_vformatMessageWithError(  const char  *locale,
-                            const UChar *pattern,
+                            const char16_t *pattern,
                             int32_t     patternLength,
-                            UChar       *result,
+                            char16_t    *result,
                             int32_t     resultLength,
                             UParseError *parseError,
                             va_list     ap,
                             UErrorCode  *status)
 
 {
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,parseError,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
@@ -144,15 +144,15 @@ u_vformatMessageWithError(  const char  *locale,
 //  3. Iterate through each formattable returned, and assign to the arguments
 U_CAPI void
 u_parseMessage( const char   *locale,
-                const UChar  *pattern,
+                const char16_t  *pattern,
                 int32_t      patternLength,
-                const UChar  *source,
+                const char16_t  *source,
                 int32_t      sourceLength,
                 UErrorCode   *status,
                 ...)
 {
     va_list    ap;
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
 
     // start vararg processing
     va_start(ap, status);
@@ -164,15 +164,15 @@ u_parseMessage( const char   *locale,
 
 U_CAPI void U_EXPORT2
 u_vparseMessage(const char  *locale,
-                const UChar *pattern,
+                const char16_t *pattern,
                 int32_t     patternLength,
-                const UChar *source,
+                const char16_t *source,
                 int32_t     sourceLength,
                 va_list     ap,
                 UErrorCode  *status)
 {
-    //argument checking defered to subsequent method calls
-    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
+    //argument checking deferred to subsequent method calls
+    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,nullptr,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
     umsg_close(fmt);
@@ -180,9 +180,9 @@ u_vparseMessage(const char  *locale,
 
 U_CAPI void
 u_parseMessageWithError(const char  *locale,
-                        const UChar *pattern,
+                        const char16_t *pattern,
                         int32_t     patternLength,
-                        const UChar *source,
+                        const char16_t *source,
                         int32_t     sourceLength,
                         UParseError *error,
                         UErrorCode  *status,
@@ -190,7 +190,7 @@ u_parseMessageWithError(const char  *locale,
 {
     va_list    ap;
 
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
 
     // start vararg processing
     va_start(ap, status);
@@ -201,15 +201,15 @@ u_parseMessageWithError(const char  *locale,
 }
 U_CAPI void U_EXPORT2
 u_vparseMessageWithError(const char  *locale,
-                         const UChar *pattern,
+                         const char16_t *pattern,
                          int32_t     patternLength,
-                         const UChar *source,
+                         const char16_t *source,
                          int32_t     sourceLength,
                          va_list     ap,
                          UParseError *error,
                          UErrorCode* status)
 {
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,error,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
@@ -223,24 +223,24 @@ u_vparseMessageWithError(const char  *locale,
 
 
 U_CAPI UMessageFormat* U_EXPORT2
-umsg_open(  const UChar     *pattern,
+umsg_open(  const char16_t  *pattern,
             int32_t         patternLength,
             const  char     *locale,
             UParseError     *parseError,
             UErrorCode      *status)
 {
     //check arguments
-    if(status==NULL || U_FAILURE(*status))
+    if(status==nullptr || U_FAILURE(*status))
     {
       return 0;
     }
-    if(pattern==NULL||patternLength<-1){
+    if(pattern==nullptr||patternLength<-1){
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return 0;
     }
 
     UParseError tErr;
-    if(parseError==NULL)
+    if(parseError==nullptr)
     {
         parseError = &tErr;
     }
@@ -249,9 +249,9 @@ umsg_open(  const UChar     *pattern,
     UnicodeString patString(patternLength == -1, pattern, len);
 
     MessageFormat* retVal = new MessageFormat(patString,Locale(locale),*parseError,*status);
-    if(retVal == NULL) {
+    if(retVal == nullptr) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
+        return nullptr;
     }
     if (U_SUCCESS(*status) && MessageFormatAdapter::hasArgTypeConflicts(*retVal)) {
         *status = U_ARGUMENT_TYPE_MISMATCH;
@@ -263,7 +263,7 @@ U_CAPI void U_EXPORT2
 umsg_close(UMessageFormat* format)
 {
     //check arguments
-    if(format==NULL){
+    if(format==nullptr){
         return;
     }
     delete (MessageFormat*) format;
@@ -274,12 +274,12 @@ umsg_clone(const UMessageFormat *fmt,
            UErrorCode *status)
 {
     //check arguments
-    if(status==NULL || U_FAILURE(*status)){
-        return NULL;
+    if(status==nullptr || U_FAILURE(*status)){
+        return nullptr;
     }
-    if(fmt==NULL){
+    if(fmt==nullptr){
         *status = U_ILLEGAL_ARGUMENT_ERROR;
-        return NULL;
+        return nullptr;
     }
     UMessageFormat retVal = (UMessageFormat)((MessageFormat*)fmt)->clone();
     if(retVal == 0) {
@@ -293,7 +293,7 @@ U_CAPI void  U_EXPORT2
 umsg_setLocale(UMessageFormat *fmt, const char* locale)
 {
     //check arguments
-    if(fmt==NULL){
+    if(fmt==nullptr){
         return;
     }
     ((MessageFormat*)fmt)->setLocale(Locale(locale));   
@@ -303,7 +303,7 @@ U_CAPI const char*  U_EXPORT2
 umsg_getLocale(const UMessageFormat *fmt)
 {
     //check arguments
-    if(fmt==NULL){
+    if(fmt==nullptr){
         return "";
     }
     return ((const MessageFormat*)fmt)->getLocale().getName();
@@ -311,22 +311,22 @@ umsg_getLocale(const UMessageFormat *fmt)
 
 U_CAPI void  U_EXPORT2
 umsg_applyPattern(UMessageFormat *fmt,
-                           const UChar* pattern,
+                           const char16_t* pattern,
                            int32_t patternLength,
                            UParseError* parseError,
                            UErrorCode* status)
 {
     //check arguments
     UParseError tErr;
-    if(status ==NULL||U_FAILURE(*status)){
+    if(status ==nullptr||U_FAILURE(*status)){
         return ;
     }
-    if(fmt==NULL || (pattern==NULL && patternLength!=0) || patternLength<-1) {
+    if(fmt==nullptr || (pattern==nullptr && patternLength!=0) || patternLength<-1) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return ;
     }
 
-    if(parseError==NULL){
+    if(parseError==nullptr){
       parseError = &tErr;
     }
 
@@ -336,23 +336,23 @@ umsg_applyPattern(UMessageFormat *fmt,
 
 U_CAPI int32_t  U_EXPORT2
 umsg_toPattern(const UMessageFormat *fmt,
-               UChar* result, 
+               char16_t* result,
                int32_t resultLength,
                UErrorCode* status)
 {
     //check arguments
-    if(status ==NULL||U_FAILURE(*status)){
+    if(status ==nullptr||U_FAILURE(*status)){
         return -1;
     }
-    if(fmt==NULL||resultLength<0 || (resultLength>0 && result==0)){
+    if(fmt==nullptr||resultLength<0 || (resultLength>0 && result==0)){
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
 
 
     UnicodeString res;
-    if(!(result==NULL && resultLength==0)) {
-        // NULL destination for pure preflighting: empty dummy string
+    if(!(result==nullptr && resultLength==0)) {
+        // nullptr destination for pure preflighting: empty dummy string
         // otherwise, alias the destination buffer
         res.setTo(result, 0, resultLength);
     }
@@ -362,15 +362,15 @@ umsg_toPattern(const UMessageFormat *fmt,
 
 U_CAPI int32_t
 umsg_format(    const UMessageFormat *fmt,
-                UChar          *result,
+                char16_t       *result,
                 int32_t        resultLength,
                 UErrorCode     *status,
                 ...)
 {
     va_list    ap;
     int32_t actLen;  
-    //argument checking defered to last method call umsg_vformat which
-    //saves time when arguments are valid and we dont care when arguments are not
+    //argument checking deferred to last method call umsg_vformat which
+    //saves time when arguments are valid and we don't care when arguments are not
     //since we return an error anyway
 
     
@@ -387,7 +387,7 @@ umsg_format(    const UMessageFormat *fmt,
 
 U_CAPI int32_t U_EXPORT2
 umsg_vformat(   const UMessageFormat *fmt,
-                UChar          *result,
+                char16_t       *result,
                 int32_t        resultLength,
                 va_list        ap,
                 UErrorCode     *status)
@@ -397,7 +397,7 @@ umsg_vformat(   const UMessageFormat *fmt,
     {
         return -1;
     }
-    if(fmt==NULL||resultLength<0 || (resultLength>0 && result==0)) {
+    if(fmt==nullptr||resultLength<0 || (resultLength>0 && result==0)) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
@@ -412,7 +412,7 @@ umsg_vformat(   const UMessageFormat *fmt,
     // iterate through the vararg list, and get the arguments out
     for(int32_t i = 0; i < count; ++i) {
         
-        UChar *stringVal;
+        char16_t *stringVal;
         double tDouble=0;
         int32_t tInt =0;
         int64_t tInt64 = 0;
@@ -440,7 +440,7 @@ umsg_vformat(   const UMessageFormat *fmt,
             
         case Formattable::kString:
             // For some reason, a temporary is needed
-            stringVal = va_arg(ap, UChar*);
+            stringVal = va_arg(ap, char16_t*);
             if(stringVal){
                 args[i].setString(UnicodeString(stringVal));
             }else{
@@ -463,7 +463,7 @@ umsg_vformat(   const UMessageFormat *fmt,
 
         default:
             // Unknown/unsupported argument type.
-            UPRV_UNREACHABLE;
+            UPRV_UNREACHABLE_EXIT;
         }
     }
     UnicodeString resultStr;
@@ -483,15 +483,15 @@ umsg_vformat(   const UMessageFormat *fmt,
 
 U_CAPI void
 umsg_parse( const UMessageFormat *fmt,
-            const UChar    *source,
+            const char16_t *source,
             int32_t        sourceLength,
             int32_t        *count,
             UErrorCode     *status,
             ...)
 {
     va_list    ap;
-    //argument checking defered to last method call umsg_vparse which
-    //saves time when arguments are valid and we dont care when arguments are not
+    //argument checking deferred to last method call umsg_vparse which
+    //saves time when arguments are valid and we don't care when arguments are not
     //since we return an error anyway
 
     // start vararg processing
@@ -505,18 +505,18 @@ umsg_parse( const UMessageFormat *fmt,
 
 U_CAPI void U_EXPORT2
 umsg_vparse(const UMessageFormat *fmt,
-            const UChar    *source,
+            const char16_t *source,
             int32_t        sourceLength,
             int32_t        *count,
             va_list        ap,
             UErrorCode     *status)
 {
     //check arguments
-    if(status==NULL||U_FAILURE(*status))
+    if(status==nullptr||U_FAILURE(*status))
     {
         return;
     }
-    if(fmt==NULL||source==NULL || sourceLength<-1 || count==NULL){
+    if(fmt==nullptr||source==nullptr || sourceLength<-1 || count==nullptr){
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return;
     }
@@ -528,7 +528,7 @@ umsg_vparse(const UMessageFormat *fmt,
     Formattable *args = ((const MessageFormat*)fmt)->parse(srcString,*count,*status);
     UDate *aDate;
     double *aDouble;
-    UChar *aString;
+    char16_t *aString;
     int32_t* aInt;
     int64_t* aInt64;
     UnicodeString temp;
@@ -574,7 +574,7 @@ umsg_vparse(const UMessageFormat *fmt,
             break;
 
         case Formattable::kString:
-            aString = va_arg(ap, UChar*);
+            aString = va_arg(ap, char16_t*);
             if(aString){
                 args[i].getString(temp);
                 len = temp.length();
@@ -590,11 +590,11 @@ umsg_vparse(const UMessageFormat *fmt,
             // support kObject.  When MessageFormat is changed to
             // understand MeasureFormats, modify this code to do the
             // right thing. [alan]
-            UPRV_UNREACHABLE;
+            UPRV_UNREACHABLE_EXIT;
 
         // better not happen!
         case Formattable::kArray:
-            UPRV_UNREACHABLE;
+            UPRV_UNREACHABLE_EXIT;
         }
     }
 
@@ -602,9 +602,9 @@ umsg_vparse(const UMessageFormat *fmt,
     delete [] args;
 }
 
-#define SINGLE_QUOTE      ((UChar)0x0027)
-#define CURLY_BRACE_LEFT  ((UChar)0x007B)
-#define CURLY_BRACE_RIGHT ((UChar)0x007D)
+#define SINGLE_QUOTE      ((char16_t)0x0027)
+#define CURLY_BRACE_LEFT  ((char16_t)0x007B)
+#define CURLY_BRACE_RIGHT ((char16_t)0x007D)
 
 #define STATE_INITIAL 0
 #define STATE_SINGLE_QUOTE 1
@@ -613,9 +613,9 @@ umsg_vparse(const UMessageFormat *fmt,
 
 #define MAppend(c) if (len < destCapacity) dest[len++] = c; else len++
 
-int32_t umsg_autoQuoteApostrophe(const UChar* pattern, 
+int32_t umsg_autoQuoteApostrophe(const char16_t* pattern,
                  int32_t patternLength,
-                 UChar* dest,
+                 char16_t* dest,
                  int32_t destCapacity,
                  UErrorCode* ec)
 {
@@ -623,11 +623,11 @@ int32_t umsg_autoQuoteApostrophe(const UChar* pattern,
     int32_t braceCount = 0;
     int32_t len = 0;
 
-    if (ec == NULL || U_FAILURE(*ec)) {
+    if (ec == nullptr || U_FAILURE(*ec)) {
         return -1;
     }
 
-    if (pattern == NULL || patternLength < -1 || (dest == NULL && destCapacity > 0)) {
+    if (pattern == nullptr || patternLength < -1 || (dest == nullptr && destCapacity > 0)) {
         *ec = U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
@@ -638,7 +638,7 @@ int32_t umsg_autoQuoteApostrophe(const UChar* pattern,
     }
 
     for (int i = 0; i < patternLength; ++i) {
-        UChar c = pattern[i];
+        char16_t c = pattern[i];
         switch (state) {
         case STATE_INITIAL:
             switch (c) {
