@@ -2,8 +2,8 @@
 
 load(libdir + "iteration.js");
 
-var map = Map([['a', 0], ['b', 1], ['c', 2], ['d', 3]]);
-var iter = map[std_iterator]();
+var map = new Map([['a', 0], ['b', 1], ['c', 2], ['d', 3]]);
+var iter = map[Symbol.iterator]();
 var log = '';
 for (let [k, v] of iter) {
     log += k + v;
