@@ -1,6 +1,6 @@
 // Basic getOwnPropertyNames tests.
 
-var g = newGlobal();
+var g = newGlobal({newCompartment: true});
 var dbg = Debugger();
 var gobj = dbg.addDebuggee(g);
 
