@@ -1,6 +1,6 @@
 // Tests that earlier try notes don't interfere with later exception handling.
 
-var g = newGlobal();
+var g = newGlobal({newCompartment: true});
 g.debuggeeGlobal = this;
 g.eval("(" + function () {
   dbg = new Debugger(debuggeeGlobal);
