@@ -1,6 +1,6 @@
 // Test that OSR respect debuggeeness.
 
-var g = newGlobal();
+var g = newGlobal({newCompartment: true});
 var dbg = new Debugger(g);
 
 g.eval("" + function f(c) {
