@@ -11,9 +11,9 @@
 #include "jit/arm64/vixl/Debugger-vixl.h"
 #include "jit/arm64/vixl/MacroAssembler-vixl.h"
 #include "jit/AtomicOp.h"
-#include "jit/JitFrames.h"
 #include "jit/MoveResolver.h"
 #include "vm/BigIntType.h"  // JS::BigInt
+#include "wasm/WasmBuiltins.h"
 
 #ifdef _M_ARM64
 #  ifdef move32
@@ -261,6 +261,39 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       Mov(GetStackPointer64(), vixl::sp);
     }
   }
+  // In debug builds only, cause a trap if PSP is active and PSP != SP
+  void assertStackPtrsSynced(uint32_t id) {
+#ifdef DEBUG
+    // The add and sub instructions below will only take a 12-bit immediate.
+    MOZ_ASSERT(id <= 0xFFF);
+    if (!GetStackPointer64().Is(vixl::sp)) {
+      Label ok;
+      // Add a marker, so we can figure out who requested the check when
+      // inspecting the generated code.  Note, a more concise way to encode
+      // the marker would be to use it as an immediate for the `brk`
+      // instruction as generated by `Unreachable()`, and removing the add/sub.
+      Add(GetStackPointer64(), GetStackPointer64(), Operand(id));
+      Sub(GetStackPointer64(), GetStackPointer64(), Operand(id));
+      Cmp(vixl::sp, GetStackPointer64());
+      B(Equal, &ok);
+      Unreachable();
+      bind(&ok);
+    }
+#endif
+  }
+  // In debug builds only, add a marker that doesn't change the machine's
+  // state.  Note these markers are x16-based, as opposed to the x28-based
+  // ones made by `assertStackPtrsSynced`.
+  void addMarker(uint32_t id) {
+#ifdef DEBUG
+    // Only 12 bits of immediate are allowed.
+    MOZ_ASSERT(id <= 0xFFF);
+    ARMRegister x16 = ARMRegister(r16, 64);
+    Add(x16, x16, Operand(id));
+    Sub(x16, x16, Operand(id));
+#endif
+  }
+
   void storeValue(ValueOperand val, const Address& dest) {
     storePtr(val.valueReg(), dest);
   }
@@ -288,6 +321,13 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     storePtr(temp, dest);
   }
 
+  void storePrivateValue(Register src, const Address& dest) {
+    storePtr(src, dest);
+  }
+  void storePrivateValue(ImmGCPtr imm, const Address& dest) {
+    storePtr(imm, dest);
+  }
+
   void loadValue(Address src, Register val) {
     Ldr(ARMRegister(val, 64), MemOperand(src));
   }
@@ -310,6 +350,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
   void popValue(ValueOperand val) {
     vixl::MacroAssembler::Pop(ARMRegister(val.valueReg(), 64));
+    // SP may be < PSP now (that's OK).
+    // eg testcase: tests/backup-point-bug1315634.js
   }
   void pushValue(const Value& val) {
     vixl::UseScratchRegisterScope temps(this);
@@ -338,6 +380,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     loadValue(addr, scratch);
     push(scratch);
   }
+  void pushValue(const BaseIndex& addr, Register scratch) {
+    loadValue(addr, ValueOperand(scratch));
+    pushValue(ValueOperand(scratch));
+  }
   template <typename T>
   void storeUnboxedPayload(ValueOperand value, T address, size_t nbytes,
                            JSValueType type) {
@@ -399,39 +445,39 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     sbfx(ARMRegister(dest, 64), ARMRegister(src, 64), JSVAL_TAG_SHIFT,
          (64 - JSVAL_TAG_SHIFT));
   }
-  MOZ_MUST_USE Register extractTag(const Address& address, Register scratch) {
+  [[nodiscard]] Register extractTag(const Address& address, Register scratch) {
     loadPtr(address, scratch);
     splitSignExtTag(scratch, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractTag(const ValueOperand& value,
-                                   Register scratch) {
+  [[nodiscard]] Register extractTag(const ValueOperand& value,
+                                    Register scratch) {
     splitSignExtTag(value.valueReg(), scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractObject(const Address& address,
-                                      Register scratch) {
+  [[nodiscard]] Register extractObject(const Address& address,
+                                       Register scratch) {
     loadPtr(address, scratch);
     unboxObject(scratch, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractObject(const ValueOperand& value,
-                                      Register scratch) {
+  [[nodiscard]] Register extractObject(const ValueOperand& value,
+                                       Register scratch) {
     unboxObject(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractSymbol(const ValueOperand& value,
-                                      Register scratch) {
+  [[nodiscard]] Register extractSymbol(const ValueOperand& value,
+                                       Register scratch) {
     unboxSymbol(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractInt32(const ValueOperand& value,
-                                     Register scratch) {
+  [[nodiscard]] Register extractInt32(const ValueOperand& value,
+                                      Register scratch) {
     unboxInt32(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractBoolean(const ValueOperand& value,
-                                       Register scratch) {
+  [[nodiscard]] Register extractBoolean(const ValueOperand& value,
+                                        Register scratch) {
     unboxBoolean(value, scratch);
     return scratch;
   }
@@ -531,9 +577,16 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     ARMFPRegister fsrc64(src, 64);
     ARMRegister dest32(dest, 32);
 
-    // ARMv8.3 chips support the FJCVTZS instruction, which handles
-    // exactly this logic.
-    if (CPUHas(vixl::CPUFeatures::kFP, vixl::CPUFeatures::kJSCVT)) {
+    // ARMv8.3 chips support the FJCVTZS instruction, which handles exactly this
+    // logic.  But the simulator does not implement it, and when the simulator
+    // runs on ARM64 hardware we want to override vixl's detection of it.
+#if defined(JS_SIMULATOR_ARM64) && (defined(__aarch64__) || defined(_M_ARM64))
+    const bool fjscvt = false;
+#else
+    const bool fjscvt =
+        CPUHas(vixl::CPUFeatures::kFP, vixl::CPUFeatures::kJSCVT);
+#endif
+    if (fjscvt) {
       // Convert double to integer, rounding toward zero.
       // The Z-flag is set iff the conversion is exact. -0 unsets the Z-flag.
       Fjcvtzs(dest32, fsrc64);
@@ -545,14 +598,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
         B(&done, Assembler::Zero);  // If conversion was exact, go to end.
 
         // The conversion was inexact, but the caller intends to allow -0.
-        vixl::UseScratchRegisterScope temps(this);
-        const ARMFPRegister scratch64 = temps.AcquireD();
-        MOZ_ASSERT(!scratch64.Is(fsrc64));
 
         // Compare fsrc64 to 0.
         // If fsrc64 == 0 and FJCVTZS conversion was inexact, then fsrc64 is -0.
-        Fmov(scratch64, xzr);
-        Fcmp(scratch64, fsrc64);
+        Fcmp(fsrc64, 0.0);
         B(fail, Assembler::NotEqual);  // Pass through -0; fail otherwise.
 
         bind(&done);
@@ -602,112 +651,46 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       Cbnz(dest32, fail);
       bind(&nonzero);
     }
-    And(dest64, dest64, Operand(0xffffffff));
+    Uxtw(dest64, dest64);
   }
 
-  void floor(FloatRegister input, Register output, Label* bail) {
-    Label handleZero;
-    // Label handleNeg;
-    Label fin;
-    ARMFPRegister iDbl(input, 64);
-    ARMRegister o64(output, 64);
-    ARMRegister o32(output, 32);
-    Fcmp(iDbl, 0.0);
-    B(Assembler::Equal, &handleZero);
-    // B(Assembler::Signed, &handleNeg);
-    // NaN is always a bail condition, just bail directly.
-    B(Assembler::Overflow, bail);
-    Fcvtms(o64, iDbl);
-    Cmp(o64, Operand(o64, vixl::SXTW));
-    B(NotEqual, bail);
-    Mov(o32, o32);
-    B(&fin);
+  void convertDoubleToPtr(FloatRegister src, Register dest, Label* fail,
+                          bool negativeZeroCheck = true) {
+    ARMFPRegister fsrc64(src, 64);
+    ARMRegister dest64(dest, 64);
 
-    bind(&handleZero);
-    // Move the top word of the double into the output reg, if it is non-zero,
-    // then the original value was -0.0.
-    Fmov(o64, iDbl);
-    Cbnz(o64, bail);
-    bind(&fin);
-  }
-
-  void floorf(FloatRegister input, Register output, Label* bail) {
-    Label handleZero;
-    // Label handleNeg;
-    Label fin;
-    ARMFPRegister iFlt(input, 32);
-    ARMRegister o64(output, 64);
-    ARMRegister o32(output, 32);
-    Fcmp(iFlt, 0.0);
-    B(Assembler::Equal, &handleZero);
-    // B(Assembler::Signed, &handleNeg);
-    // NaN is always a bail condition, just bail directly.
-    B(Assembler::Overflow, bail);
-    Fcvtms(o64, iFlt);
-    Cmp(o64, Operand(o64, vixl::SXTW));
-    B(NotEqual, bail);
-    Mov(o32, o32);
-    B(&fin);
-
-    bind(&handleZero);
-    // Move the top word of the double into the output reg, if it is non-zero,
-    // then the original value was -0.0.
-    Fmov(o32, iFlt);
-    Cbnz(o32, bail);
-    bind(&fin);
-  }
-
-  void ceil(FloatRegister input, Register output, Label* bail) {
-    Label handleZero;
-    Label fin;
-    ARMFPRegister iDbl(input, 64);
-    ARMRegister o64(output, 64);
-    ARMRegister o32(output, 32);
-    Fcmp(iDbl, 0.0);
-    B(Assembler::Overflow, bail);
-    Fcvtps(o64, iDbl);
-    Cmp(o64, Operand(o64, vixl::SXTW));
-    B(NotEqual, bail);
-    Cbz(o64, &handleZero);
-    Mov(o32, o32);
-    B(&fin);
-
-    bind(&handleZero);
     vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch = temps.AcquireX();
-    Fmov(scratch, iDbl);
-    Cbnz(scratch, bail);
-    bind(&fin);
-  }
+    const ARMFPRegister scratch64 = temps.AcquireD();
+    MOZ_ASSERT(!scratch64.Is(fsrc64));
 
-  void ceilf(FloatRegister input, Register output, Label* bail) {
-    Label handleZero;
-    Label fin;
-    ARMFPRegister iFlt(input, 32);
-    ARMRegister o64(output, 64);
-    ARMRegister o32(output, 32);
-    Fcmp(iFlt, 0.0);
+    // Note: we can't use the FJCVTZS instruction here because that only works
+    // for 32-bit values.
 
-    // NaN is always a bail condition, just bail directly.
-    B(Assembler::Overflow, bail);
-    Fcvtps(o64, iFlt);
-    Cmp(o64, Operand(o64, vixl::SXTW));
-    B(NotEqual, bail);
-    Cbz(o64, &handleZero);
-    Mov(o32, o32);
-    B(&fin);
+    Fcvtzs(dest64, fsrc64);    // Convert, rounding toward zero.
+    Scvtf(scratch64, dest64);  // Convert back, using FPCR rounding mode.
+    Fcmp(scratch64, fsrc64);
+    B(fail, Assembler::NotEqual);
 
-    bind(&handleZero);
-    // Move the top word of the double into the output reg, if it is non-zero,
-    // then the original value was -0.0.
-    Fmov(o32, iFlt);
-    Cbnz(o32, bail);
-    bind(&fin);
+    if (negativeZeroCheck) {
+      Label nonzero;
+      Cbnz(dest64, &nonzero);
+      Fmov(dest64, fsrc64);
+      Cbnz(dest64, fail);
+      bind(&nonzero);
+    }
   }
 
   void jump(Label* label) { B(label); }
   void jump(JitCode* code) { branch(code); }
   void jump(ImmPtr ptr) {
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/asm.js/testBug1046688.js.
+    // * Removing the sync causes no failures in all of jit-tests.
+    //
+    // Also see branch(JitCode*) below. This version of jump() is called only
+    // from jump(TrampolinePtr) which is called on various very slow paths,
+    // probably only in JS.
     syncStackPtr();
     BufferOffset loc =
         b(-1,
@@ -717,8 +700,12 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void jump(TrampolinePtr code) { jump(ImmPtr(code.value)); }
   void jump(Register reg) { Br(ARMRegister(reg, 64)); }
   void jump(const Address& addr) {
-    loadPtr(addr, ip0);
-    Br(vixl::ip0);
+    vixl::UseScratchRegisterScope temps(this);
+    MOZ_ASSERT(temps.IsAvailable(ScratchReg64));  // ip0
+    temps.Exclude(ScratchReg64);
+    MOZ_ASSERT(addr.base != ScratchReg64.asUnsized());
+    loadPtr(addr, ScratchReg64.asUnsized());
+    br(ScratchReg64);
   }
 
   void align(int alignment) { armbuffer_.align(alignment); }
@@ -930,16 +917,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     doBaseIndex(ARMRegister(r, 32), address, vixl::STR_w);
   }
 
-  void store32_NoSecondScratch(Imm32 imm, const Address& address) {
-    vixl::UseScratchRegisterScope temps(this);
-    temps.Exclude(ARMRegister(ScratchReg2, 32));  // Disallow ScratchReg2.
-    const ARMRegister scratch32 = temps.AcquireW();
-
-    MOZ_ASSERT(scratch32.asUnsized() != address.base);
-    Mov(scratch32, uint64_t(imm.value));
-    Str(scratch32, toMemOperand(address));
-  }
-
   template <typename S, typename T>
   void store32Unaligned(const S& src, const T& dest) {
     store32(src, dest);
@@ -955,6 +932,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     storePtr(ImmWord(imm.value), address);
   }
 
+  void store64(Imm64 imm, const Address& address) {
+    storePtr(ImmWord(imm.value), address);
+  }
+
   template <typename S, typename T>
   void store64Unaligned(const S& src, const T& dest) {
     store64(src, dest);
@@ -971,7 +952,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   inline void subStackPtrFrom(Register dest);
 
   inline void andToStackPtr(Imm32 t);
-  inline void andStackPtrTo(Register dest);
 
   inline void moveToStackPtr(Register src);
   inline void moveStackPtrTo(Register dest);
@@ -1023,14 +1003,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Ldr(scratch32, toMemOperand(lhs));
     Cmp(scratch32, Operand(ARMRegister(rhs, 32)));
   }
-  void cmp32(Register lhs, const Address& rhs) {
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch32 = temps.AcquireW();
-    MOZ_ASSERT(scratch32.asUnsized() != rhs.base);
-    MOZ_ASSERT(scratch32.asUnsized() != lhs);
-    Ldr(scratch32, toMemOperand(rhs));
-    Cmp(scratch32, Operand(ARMRegister(lhs, 32)));
-  }
   void cmp32(const vixl::Operand& lhs, Imm32 rhs) {
     vixl::UseScratchRegisterScope temps(this);
     const ARMRegister scratch32 = temps.AcquireW();
@@ -1043,12 +1015,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Mov(scratch32, lhs);
     Cmp(scratch32, Operand(ARMRegister(rhs, 32)));
   }
-  void cmp32(Register lhs, const vixl::Operand& rhs) {
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch32 = temps.AcquireW();
-    Mov(scratch32, rhs);
-    Cmp(scratch32, Operand(ARMRegister(lhs, 32)));
-  }
 
   void cmn32(Register lhs, Imm32 rhs) {
     Cmn(ARMRegister(lhs, 32), Operand(rhs.value));
@@ -1063,6 +1029,9 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void cmpPtr(Register lhs, ImmPtr rhs) {
     Cmp(ARMRegister(lhs, 64), Operand(uint64_t(rhs.value)));
   }
+  void cmpPtr(Register lhs, Imm64 rhs) {
+    Cmp(ARMRegister(lhs, 64), Operand(uint64_t(rhs.value)));
+  }
   void cmpPtr(Register lhs, Register rhs) {
     Cmp(ARMRegister(lhs, 64), ARMRegister(rhs, 64));
   }
@@ -1184,6 +1153,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     MOZ_CRASH("moveFloatAsDouble");
   }
 
+  void moveSimd128(FloatRegister src, FloatRegister dest) {
+    fmov(ARMFPRegister(dest, 128), ARMFPRegister(src, 128));
+  }
+
   void splitSignExtTag(const ValueOperand& operand, Register dest) {
     splitSignExtTag(operand.valueReg(), dest);
   }
@@ -1279,6 +1252,16 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Adds(scratch32, scratch32, Operand(imm.value));
     Str(scratch32, toMemOperand(dest));
   }
+  void adds64(Imm32 imm, Register dest) {
+    Adds(ARMRegister(dest, 64), ARMRegister(dest, 64), Operand(imm.value));
+  }
+  void adds64(ImmWord imm, Register dest) {
+    Adds(ARMRegister(dest, 64), ARMRegister(dest, 64), Operand(imm.value));
+  }
+  void adds64(Register src, Register dest) {
+    Adds(ARMRegister(dest, 64), ARMRegister(dest, 64),
+         Operand(ARMRegister(src, 64)));
+  }
 
   void subs32(Imm32 imm, Register dest) {
     Subs(ARMRegister(dest, 32), ARMRegister(dest, 32), Operand(imm.value));
@@ -1287,6 +1270,17 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Subs(ARMRegister(dest, 32), ARMRegister(dest, 32),
          Operand(ARMRegister(src, 32)));
   }
+  void subs64(Imm32 imm, Register dest) {
+    Subs(ARMRegister(dest, 64), ARMRegister(dest, 64), Operand(imm.value));
+  }
+  void subs64(Register src, Register dest) {
+    Subs(ARMRegister(dest, 64), ARMRegister(dest, 64),
+         Operand(ARMRegister(src, 64)));
+  }
+
+  void negs32(Register reg) {
+    Negs(ARMRegister(reg, 32), Operand(ARMRegister(reg, 32)));
+  }
 
   void ret() {
     pop(lr);
@@ -1294,17 +1288,27 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
   void retn(Imm32 n) {
+    vixl::UseScratchRegisterScope temps(this);
+    MOZ_ASSERT(temps.IsAvailable(ScratchReg64));  // ip0
+    temps.Exclude(ScratchReg64);
     // ip0 <- [sp]; sp += n; ret ip0
-    Ldr(vixl::ip0,
+    Ldr(ScratchReg64,
         MemOperand(GetStackPointer64(), ptrdiff_t(n.value), vixl::PostIndex));
     syncStackPtr();  // SP is always used to transmit the stack between calls.
-    Ret(vixl::ip0);
+    Ret(ScratchReg64);
   }
 
   void j(Condition cond, Label* dest) { B(dest, cond); }
 
   void branch(Condition cond, Label* label) { B(label, cond); }
   void branch(JitCode* target) {
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/async/debugger-reject-after-fulfill.js
+    // * Removing the sync causes no failures in all of jit-tests.
+    //
+    // Also see jump() above.  This is used only to implement jump(JitCode*)
+    // and only for JS, it appears.
     syncStackPtr();
     BufferOffset loc =
         b(-1,
@@ -1322,6 +1326,19 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Fcmp(ARMFPRegister(lhs, 32), ARMFPRegister(rhs, 32));
   }
 
+  void compareSimd128Int(Assembler::Condition cond, ARMFPRegister dest,
+                         ARMFPRegister lhs, ARMFPRegister rhs);
+  void compareSimd128Float(Assembler::Condition cond, ARMFPRegister dest,
+                           ARMFPRegister lhs, ARMFPRegister rhs);
+  void rightShiftInt8x16(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt16x8(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt32x4(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt64x2(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+
   void branchNegativeZero(FloatRegister reg, Register scratch, Label* label) {
     MOZ_CRASH("branchNegativeZero");
   }
@@ -1432,6 +1449,17 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
         Operand(JS::detail::ValueGCThingPayloadMask));
   }
 
+  // Like unboxGCThingForGCBarrier, but loads the GC thing's chunk base.
+  void getGCThingValueChunk(const Address& src, Register dest) {
+    loadPtr(src, dest);
+    And(ARMRegister(dest, 64), ARMRegister(dest, 64),
+        Operand(JS::detail::ValueGCThingPayloadChunkMask));
+  }
+  void getGCThingValueChunk(const ValueOperand& src, Register dest) {
+    And(ARMRegister(dest, 64), ARMRegister(src.valueReg(), 64),
+        Operand(JS::detail::ValueGCThingPayloadChunkMask));
+  }
+
   inline void unboxValue(const ValueOperand& src, AnyRegister dest,
                          JSValueType type);
 
@@ -1469,10 +1497,31 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
   void loadConstantDouble(double d, FloatRegister dest) {
-    Fmov(ARMFPRegister(dest, 64), d);
+    ARMFPRegister r(dest, 64);
+    if (d == 0.0) {
+      // Clang11 does movi for 0 and movi+fneg for -0, and this seems like a
+      // good implementation-independent strategy as it avoids any gpr->fpr
+      // moves or memory traffic.
+      Movi(r, 0);
+      if (std::signbit(d)) {
+        Fneg(r, r);
+      }
+    } else {
+      Fmov(r, d);
+    }
   }
   void loadConstantFloat32(float f, FloatRegister dest) {
-    Fmov(ARMFPRegister(dest, 32), f);
+    ARMFPRegister r(dest, 32);
+    if (f == 0.0) {
+      // See comments above.  There's not a movi variant for a single register,
+      // so clear the double.
+      Movi(ARMFPRegister(dest, 64), 0);
+      if (std::signbit(f)) {
+        Fneg(r, r);
+      }
+    } else {
+      Fmov(r, f);
+    }
   }
 
   void cmpTag(Register tag, ImmTag ref) {
@@ -1814,17 +1863,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     splitSignExtTag(src, scratch);
     return testBigInt(cond, scratch);
   }
-  Condition testBigIntTruthy(bool truthy, const ValueOperand& value) {
-    vixl::UseScratchRegisterScope temps(this);
-    const Register scratch = temps.AcquireX().asUnsized();
-
-    MOZ_ASSERT(value.valueReg() != scratch);
-
-    unboxBigInt(value, scratch);
-    load32(Address(scratch, BigInt::offsetOfDigitLength()), scratch);
-    cmp32(scratch, Imm32(0));
-    return truthy ? Condition::NonZero : Condition::Zero;
-  }
   Condition testInt32(Condition cond, const BaseIndex& src) {
     vixl::UseScratchRegisterScope temps(this);
     const Register scratch = temps.AcquireX().asUnsized();
@@ -1877,19 +1915,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Tst(payload32, payload32);
     return truthy ? NonZero : Zero;
   }
-  Condition testStringTruthy(bool truthy, const ValueOperand& value) {
-    vixl::UseScratchRegisterScope temps(this);
-    const Register scratch = temps.AcquireX().asUnsized();
-    const ARMRegister scratch32(scratch, 32);
-    const ARMRegister scratch64(scratch, 64);
 
-    MOZ_ASSERT(value.valueReg() != scratch);
+  Condition testBigIntTruthy(bool truthy, const ValueOperand& value);
+  Condition testStringTruthy(bool truthy, const ValueOperand& value);
 
-    unboxString(value, scratch);
-    Ldr(scratch32, MemOperand(scratch64, JSString::offsetOfLength()));
-    Cmp(scratch32, Operand(0));
-    return truthy ? Condition::NonZero : Condition::Zero;
-  }
   void int32OrDouble(Register src, ARMFPRegister dest) {
     Label isInt32;
     Label join;
@@ -1910,8 +1939,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       MOZ_ASSERT(scratch64.asUnsized() != address.base);
       Ldr(scratch64, toMemOperand(address));
       int32OrDouble(scratch64.asUnsized(), ARMFPRegister(dest.fpu(), 64));
-    } else if (type == MIRType::ObjectOrNull) {
-      unboxObjectOrNull(address, dest.gpr());
     } else {
       unboxNonDouble(address, dest.gpr(), ValueTypeFromMIRType(type));
     }
@@ -1925,17 +1952,11 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       MOZ_ASSERT(scratch64.asUnsized() != address.index);
       doBaseIndex(scratch64, address, vixl::LDR_x);
       int32OrDouble(scratch64.asUnsized(), ARMFPRegister(dest.fpu(), 64));
-    } else if (type == MIRType::ObjectOrNull) {
-      unboxObjectOrNull(address, dest.gpr());
     } else {
       unboxNonDouble(address, dest.gpr(), ValueTypeFromMIRType(type));
     }
   }
 
-  void loadInstructionPointerAfterCall(Register dest) {
-    MOZ_CRASH("loadInstructionPointerAfterCall");
-  }
-
   // Emit a B that can be toggled to a CMP. See ToggleToJmp(), ToggleToCmp().
   CodeOffset toggledJump(Label* label) {
     BufferOffset offset = b(label, Always);
@@ -1986,22 +2007,28 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
  public:
-  void handleFailureWithHandlerTail(void* handler, Label* profilerExitTail);
+  void handleFailureWithHandlerTail(Label* profilerExitTail,
+                                    Label* bailoutTail);
 
   void profilerEnterFrame(Register framePtr, Register scratch);
-  void profilerEnterFrame(RegisterOrSP framePtr, Register scratch);
-  void profilerExitFrame() {
-    jump(GetJitContext()->runtime->jitRuntime()->getProfilerExitFrameTail());
-  }
-  Address ToPayload(Address value) { return value; }
-  Address ToType(Address value) { return value; }
+  void profilerExitFrame();
 
   void wasmLoadImpl(const wasm::MemoryAccessDesc& access, Register memoryBase,
-                    Register ptr, Register ptrScratch, AnyRegister outany,
-                    Register64 out64);
+                    Register ptr, AnyRegister outany, Register64 out64);
+  void wasmLoadImpl(const wasm::MemoryAccessDesc& access, MemOperand srcAddr,
+                    AnyRegister outany, Register64 out64);
   void wasmStoreImpl(const wasm::MemoryAccessDesc& access, AnyRegister valany,
-                     Register64 val64, Register memoryBase, Register ptr,
-                     Register ptrScratch);
+                     Register64 val64, Register memoryBase, Register ptr);
+  void wasmStoreImpl(const wasm::MemoryAccessDesc& access, MemOperand destAddr,
+                     AnyRegister valany, Register64 val64);
+  // The complete address is in `address`, and `access` is used for its type
+  // attributes only; its `offset` is ignored.
+  void wasmLoadAbsolute(const wasm::MemoryAccessDesc& access,
+                        Register memoryBase, uint64_t address, AnyRegister out,
+                        Register64 out64);
+  void wasmStoreAbsolute(const wasm::MemoryAccessDesc& access,
+                         AnyRegister value, Register64 value64,
+                         Register memoryBase, uint64_t address);
 
   // Emit a BLR or NOP instruction. ToggleCall can be used to patch
   // this instruction.
@@ -2010,6 +2037,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     // for the debugger to match offset with Baseline's pcMappingEntries_.
     BufferOffset offset = nextOffset();
 
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/cacheir/bug1448136.js
+    // * Removing the sync causes no failures in all of jit-tests.
     syncStackPtr();
 
     BufferOffset loadOffset;
@@ -2086,12 +2117,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     vixl::MacroAssembler::Ret(vixl::lr);
   }
 
-  void clampCheck(Register r, Label* handleNotAnInt) {
-    MOZ_CRASH("clampCheck");
-  }
-
-  void stackCheck(ImmWord limitAddr, Label* label) { MOZ_CRASH("stackCheck"); }
-
   void incrementInt32Value(const Address& addr) {
     vixl::UseScratchRegisterScope temps(this);
     const ARMRegister scratch32 = temps.AcquireW();
@@ -2119,26 +2144,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 #endif
   }
 
-  void loadWasmGlobalPtr(uint32_t globalDataOffset, Register dest) {
-    loadPtr(Address(WasmTlsReg,
-                    offsetof(wasm::TlsData, globalArea) + globalDataOffset),
-            dest);
-  }
-  void loadWasmPinnedRegsFromTls() {
-    loadPtr(Address(WasmTlsReg, offsetof(wasm::TlsData, memoryBase)), HeapReg);
-  }
-
-  // Overwrites the payload bits of a dest register containing a Value.
-  void movePayload(Register src, Register dest) {
-    // Bfxil cannot be used with the zero register as a source.
-    if (src == rzr) {
-      And(ARMRegister(dest, 64), ARMRegister(dest, 64),
-          Operand(JS::detail::ValueTagMask));
-    } else {
-      Bfxil(ARMRegister(dest, 64), ARMRegister(src, 64), 0, JSVAL_TAG_SHIFT);
-    }
-  }
-
  protected:
   bool buildOOLFakeExitFrame(void* fakeReturnAddr);
 };
