@@ -6,18 +6,23 @@
 
 #include "shell/ModuleLoader.h"
 
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/TextUtils.h"
 
+#include "jsapi.h"
 #include "NamespaceImports.h"
 
+#include "builtin/TestingUtility.h"  // js::CreateScriptPrivate
+#include "js/Conversions.h"
+#include "js/MapAndSet.h"
 #include "js/Modules.h"
+#include "js/PropertyAndElement.h"  // JS_DefineProperty, JS_GetProperty
 #include "js/SourceText.h"
 #include "js/StableStringChars.h"
 #include "shell/jsshell.h"
 #include "shell/OSObject.h"
 #include "shell/StringUtils.h"
+#include "util/Text.h"
 #include "vm/JSAtom.h"
 #include "vm/JSContext.h"
 #include "vm/StringType.h"
@@ -27,21 +32,21 @@ using namespace js::shell;
 
 static constexpr char16_t JavaScriptScheme[] = u"javascript:";
 
-static bool IsJavaScriptURL(HandleLinearString path) {
+static bool IsJavaScriptURL(Handle<JSLinearString*> path) {
   return StringStartsWith(path, JavaScriptScheme);
 }
 
 static JSString* ExtractJavaScriptURLSource(JSContext* cx,
-                                            HandleLinearString path) {
+                                            Handle<JSLinearString*> path) {
   MOZ_ASSERT(IsJavaScriptURL(path));
 
-  const size_t schemeLength = mozilla::ArrayLength(JavaScriptScheme) - 1;
+  const size_t schemeLength = js_strlen(JavaScriptScheme);
   return SubString(cx, path, schemeLength);
 }
 
 bool ModuleLoader::init(JSContext* cx, HandleString loadPath) {
-  loadPathStr = AtomizeString(cx, loadPath, PinAtom);
-  if (!loadPathStr) {
+  loadPathStr = AtomizeString(cx, loadPath);
+  if (!loadPathStr || !PinAtom(cx, loadPathStr)) {
     return false;
   }
 
@@ -49,7 +54,7 @@ bool ModuleLoader::init(JSContext* cx, HandleString loadPath) {
 
   char16_t sep = PathSeparator;
   pathSeparatorStr = AtomizeChars(cx, &sep, 1);
-  if (!pathSeparatorStr) {
+  if (!pathSeparatorStr || !PinAtom(cx, pathSeparatorStr)) {
     return false;
   }
 
@@ -58,16 +63,21 @@ bool ModuleLoader::init(JSContext* cx, HandleString loadPath) {
   JS::SetModuleMetadataHook(rt, ModuleLoader::GetImportMetaProperties);
   JS::SetModuleDynamicImportHook(rt, ModuleLoader::ImportModuleDynamically);
 
+  JS::ImportAssertionVector assertions;
+  MOZ_ALWAYS_TRUE(assertions.reserve(1));
+  assertions.infallibleAppend(JS::ImportAssertion::Type);
+  JS::SetSupportedImportAssertions(rt, assertions);
+
   return true;
 }
 
 // static
 JSObject* ModuleLoader::ResolveImportedModule(
     JSContext* cx, JS::HandleValue referencingPrivate,
-    JS::HandleString specifier) {
+    JS::HandleObject moduleRequest) {
   ShellContext* scx = GetShellContext(cx);
   return scx->moduleLoader->resolveImportedModule(cx, referencingPrivate,
-                                                  specifier);
+                                                  moduleRequest);
 }
 
 // static
@@ -78,33 +88,113 @@ bool ModuleLoader::GetImportMetaProperties(JSContext* cx,
   return scx->moduleLoader->populateImportMeta(cx, privateValue, metaObject);
 }
 
+bool ModuleLoader::ImportMetaResolve(JSContext* cx, unsigned argc, Value* vp) {
+  CallArgs args = CallArgsFromVp(argc, vp);
+  RootedValue modulePrivate(
+      cx, js::GetFunctionNativeReserved(&args.callee(), ModulePrivateSlot));
+
+  // https://html.spec.whatwg.org/#hostgetimportmetaproperties
+  // Step 4.1. Set specifier to ? ToString(specifier).
+  //
+  // https://tc39.es/ecma262/#sec-tostring
+  RootedValue v(cx, args.get(ImportMetaResolveSpecifierArg));
+  RootedString specifier(cx, JS::ToString(cx, v));
+  if (!specifier) {
+    return false;
+  }
+
+  // Step 4.2, 4.3 are implemented in importMetaResolve.
+  ShellContext* scx = GetShellContext(cx);
+  RootedString url(cx);
+  if (!scx->moduleLoader->importMetaResolve(cx, modulePrivate, specifier,
+                                            &url)) {
+    return false;
+  }
+
+  // Step 4.4. Return the serialization of url.
+  args.rval().setString(url);
+  return true;
+}
+
 // static
 bool ModuleLoader::ImportModuleDynamically(JSContext* cx,
                                            JS::HandleValue referencingPrivate,
-                                           JS::HandleString specifier,
+                                           JS::HandleObject moduleRequest,
                                            JS::HandleObject promise) {
   ShellContext* scx = GetShellContext(cx);
-  return scx->moduleLoader->dynamicImport(cx, referencingPrivate, specifier,
+  return scx->moduleLoader->dynamicImport(cx, referencingPrivate, moduleRequest,
                                           promise);
 }
 
-bool ModuleLoader::loadRootModule(JSContext* cx, HandleString path) {
-  return loadAndExecute(cx, path);
+// static
+bool ModuleLoader::GetSupportedImportAssertions(
+    JSContext* cx, JS::ImportAssertionVector& values) {
+  MOZ_ASSERT(values.empty());
+
+  if (!values.reserve(1)) {
+    ReportOutOfMemory(cx);
+    return false;
+  }
+
+  values.infallibleAppend(JS::ImportAssertion::Type);
+
+  return true;
 }
 
-bool ModuleLoader::loadAndExecute(JSContext* cx, HandleString path) {
+bool ModuleLoader::loadRootModule(JSContext* cx, HandleString path) {
+  RootedValue rval(cx);
+  if (!loadAndExecute(cx, path, &rval)) {
+    return false;
+  }
+
+  RootedObject evaluationPromise(cx, &rval.toObject());
+  if (evaluationPromise == nullptr) {
+    return false;
+  }
+
+  return JS::ThrowOnModuleEvaluationFailure(cx, evaluationPromise);
+}
+
+bool ModuleLoader::registerTestModule(JSContext* cx, HandleObject moduleRequest,
+                                      Handle<ModuleObject*> module) {
+  Rooted<JSLinearString*> path(
+      cx, resolve(cx, moduleRequest, UndefinedHandleValue));
+  if (!path) {
+    return false;
+  }
+
+  path = normalizePath(cx, path);
+  if (!path) {
+    return false;
+  }
+
+  return addModuleToRegistry(cx, path, module);
+}
+
+void ModuleLoader::clearModules(JSContext* cx) {
+  Handle<GlobalObject*> global = cx->global();
+  global->setReservedSlot(GlobalAppSlotModuleRegistry, UndefinedValue());
+}
+
+bool ModuleLoader::loadAndExecute(JSContext* cx, HandleString path,
+                                  MutableHandleValue rval) {
   RootedObject module(cx, loadAndParse(cx, path));
   if (!module) {
     return false;
   }
 
-  return JS::ModuleInstantiate(cx, module) && JS::ModuleEvaluate(cx, module);
+  if (!JS::ModuleLink(cx, module)) {
+    return false;
+  }
+
+  return JS::ModuleEvaluate(cx, module, rval);
 }
 
 JSObject* ModuleLoader::resolveImportedModule(
     JSContext* cx, JS::HandleValue referencingPrivate,
-    JS::HandleString specifier) {
-  RootedLinearString path(cx, resolve(cx, specifier, referencingPrivate));
+    JS::HandleObject moduleRequest) {
+  Rooted<JSLinearString*> path(cx,
+                               resolve(cx, moduleRequest, referencingPrivate));
   if (!path) {
     return nullptr;
   }
@@ -115,7 +205,7 @@ JSObject* ModuleLoader::resolveImportedModule(
 bool ModuleLoader::populateImportMeta(JSContext* cx,
                                       JS::HandleValue privateValue,
                                       JS::HandleObject metaObject) {
-  RootedLinearString path(cx);
+  Rooted<JSLinearString*> path(cx);
   if (!privateValue.isUndefined()) {
     if (!getScriptPath(cx, privateValue, &path)) {
       return false;
@@ -130,12 +220,40 @@ bool ModuleLoader::populateImportMeta(JSContext* cx,
   }
 
   RootedValue pathValue(cx, StringValue(path));
-  return JS_DefineProperty(cx, metaObject, "url", pathValue, JSPROP_ENUMERATE);
+  if (!JS_DefineProperty(cx, metaObject, "url", pathValue, JSPROP_ENUMERATE)) {
+    return false;
+  }
+
+  JSFunction* resolveFunc = js::DefineFunctionWithReserved(
+      cx, metaObject, "resolve", ImportMetaResolve, ImportMetaResolveNumArgs,
+      JSPROP_ENUMERATE);
+  if (!resolveFunc) {
+    return false;
+  }
+
+  RootedObject resolveFuncObj(cx, JS_GetFunctionObject(resolveFunc));
+  js::SetFunctionNativeReserved(resolveFuncObj, ModulePrivateSlot,
+                                privateValue);
+
+  return true;
+}
+
+bool ModuleLoader::importMetaResolve(JSContext* cx,
+                                     JS::Handle<JS::Value> referencingPrivate,
+                                     JS::Handle<JSString*> specifier,
+                                     JS::MutableHandle<JSString*> urlOut) {
+  Rooted<JSLinearString*> path(cx, resolve(cx, specifier, referencingPrivate));
+  if (!path) {
+    return false;
+  }
+
+  urlOut.set(path);
+  return true;
 }
 
 bool ModuleLoader::dynamicImport(JSContext* cx,
                                  JS::HandleValue referencingPrivate,
-                                 JS::HandleString specifier,
+                                 JS::HandleObject moduleRequest,
                                  JS::HandleObject promise) {
   // To make this more realistic, use a promise to delay the import and make it
   // happen asynchronously. This method packages up the arguments and creates a
@@ -143,13 +261,13 @@ bool ModuleLoader::dynamicImport(JSContext* cx,
   // original arguments.
 
   MOZ_ASSERT(promise);
-  RootedValue specifierValue(cx, StringValue(specifier));
+  RootedValue moduleRequestValue(cx, ObjectValue(*moduleRequest));
   RootedValue promiseValue(cx, ObjectValue(*promise));
-  RootedObject closure(cx, JS_NewPlainObject(cx));
+  RootedObject closure(cx, JS_NewObjectWithGivenProto(cx, nullptr, nullptr));
   if (!closure ||
       !JS_DefineProperty(cx, closure, "referencingPrivate", referencingPrivate,
                          JSPROP_ENUMERATE) ||
-      !JS_DefineProperty(cx, closure, "specifier", specifierValue,
+      !JS_DefineProperty(cx, closure, "moduleRequest", moduleRequestValue,
                          JSPROP_ENUMERATE) ||
       !JS_DefineProperty(cx, closure, "promise", promiseValue,
                          JSPROP_ENUMERATE)) {
@@ -184,20 +302,20 @@ bool ModuleLoader::DynamicImportDelayFulfilled(JSContext* cx, unsigned argc,
   RootedObject closure(cx, &args[0].toObject());
 
   RootedValue referencingPrivate(cx);
-  RootedValue specifierValue(cx);
+  RootedValue moduleRequestValue(cx);
   RootedValue promiseValue(cx);
   if (!JS_GetProperty(cx, closure, "referencingPrivate", &referencingPrivate) ||
-      !JS_GetProperty(cx, closure, "specifier", &specifierValue) ||
+      !JS_GetProperty(cx, closure, "moduleRequest", &moduleRequestValue) ||
       !JS_GetProperty(cx, closure, "promise", &promiseValue)) {
     return false;
   }
 
-  RootedString specifier(cx, specifierValue.toString());
+  RootedObject moduleRequest(cx, &moduleRequestValue.toObject());
   RootedObject promise(cx, &promiseValue.toObject());
 
   ShellContext* scx = GetShellContext(cx);
-  return scx->moduleLoader->doDynamicImport(cx, referencingPrivate, specifier,
-                                            promise);
+  return scx->moduleLoader->doDynamicImport(cx, referencingPrivate,
+                                            moduleRequest, promise);
 }
 
 bool ModuleLoader::DynamicImportDelayRejected(JSContext* cx, unsigned argc,
@@ -207,37 +325,55 @@ bool ModuleLoader::DynamicImportDelayRejected(JSContext* cx, unsigned argc,
 
 bool ModuleLoader::doDynamicImport(JSContext* cx,
                                    JS::HandleValue referencingPrivate,
-                                   JS::HandleString specifier,
+                                   JS::HandleObject moduleRequest,
                                    JS::HandleObject promise) {
   // Exceptions during dynamic import are handled by calling
   // FinishDynamicModuleImport with a pending exception on the context.
-  mozilla::DebugOnly<bool> ok =
-      tryDynamicImport(cx, referencingPrivate, specifier, promise);
-  MOZ_ASSERT_IF(!ok, JS_IsExceptionPending(cx));
-  return JS::FinishDynamicModuleImport(cx, referencingPrivate, specifier,
-                                       promise);
+  RootedValue rval(cx);
+  bool ok =
+      tryDynamicImport(cx, referencingPrivate, moduleRequest, promise, &rval);
+  JSObject* evaluationObject = ok ? &rval.toObject() : nullptr;
+  RootedObject evaluationPromise(cx, evaluationObject);
+  return JS::FinishDynamicModuleImport(
+      cx, evaluationPromise, referencingPrivate, moduleRequest, promise);
 }
 
 bool ModuleLoader::tryDynamicImport(JSContext* cx,
                                     JS::HandleValue referencingPrivate,
-                                    JS::HandleString specifier,
-                                    JS::HandleObject promise) {
-  RootedLinearString path(cx, resolve(cx, specifier, referencingPrivate));
+                                    JS::HandleObject moduleRequest,
+                                    JS::HandleObject promise,
+                                    JS::MutableHandleValue rval) {
+  Rooted<JSLinearString*> path(cx,
+                               resolve(cx, moduleRequest, referencingPrivate));
   if (!path) {
     return false;
   }
 
-  return loadAndExecute(cx, path);
+  return loadAndExecute(cx, path, rval);
 }
 
-JSLinearString* ModuleLoader::resolve(JSContext* cx, HandleString nameArg,
+JSLinearString* ModuleLoader::resolve(JSContext* cx,
+                                      HandleObject moduleRequestArg,
                                       HandleValue referencingInfo) {
-  if (nameArg->length() == 0) {
+  ModuleRequestObject* moduleRequest =
+      &moduleRequestArg->as<ModuleRequestObject>();
+  if (moduleRequest->specifier()->length() == 0) {
     JS_ReportErrorASCII(cx, "Invalid module specifier");
     return nullptr;
   }
 
-  RootedLinearString name(cx, JS_EnsureLinearString(cx, nameArg));
+  Rooted<JSLinearString*> name(
+      cx, JS_EnsureLinearString(cx, moduleRequest->specifier()));
+  if (!name) {
+    return nullptr;
+  }
+
+  return resolve(cx, name, referencingInfo);
+}
+
+JSLinearString* ModuleLoader::resolve(JSContext* cx, HandleString specifier,
+                                      HandleValue referencingInfo) {
+  Rooted<JSLinearString*> name(cx, JS_EnsureLinearString(cx, specifier));
   if (!name) {
     return nullptr;
   }
@@ -262,7 +398,7 @@ JSLinearString* ModuleLoader::resolve(JSContext* cx, HandleString nameArg,
       return nullptr;
     }
 
-    RootedLinearString refPath(cx);
+    Rooted<JSLinearString*> refPath(cx);
     if (!getScriptPath(cx, referencingInfo, &refPath)) {
       return nullptr;
     }
@@ -296,11 +432,15 @@ JSLinearString* ModuleLoader::resolve(JSContext* cx, HandleString nameArg,
     return nullptr;
   }
 
-  return JS_EnsureLinearString(cx, result);
+  Rooted<JSLinearString*> linear(cx, JS_EnsureLinearString(cx, result));
+  if (!linear) {
+    return nullptr;
+  }
+  return normalizePath(cx, linear);
 }
 
 JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
-  RootedLinearString path(cx, JS_EnsureLinearString(cx, pathArg));
+  Rooted<JSLinearString*> path(cx, JS_EnsureLinearString(cx, pathArg));
   if (!path) {
     return nullptr;
   }
@@ -319,7 +459,7 @@ JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
     return module;
   }
 
-  UniqueChars filename = JS_EncodeStringToLatin1(cx, path);
+  UniqueChars filename = JS_EncodeStringToUTF8(cx, path);
   if (!filename) {
     return nullptr;
   }
@@ -332,15 +472,13 @@ JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
     return nullptr;
   }
 
-  JS::AutoStableStringChars stableChars(cx);
-  if (!stableChars.initTwoByte(cx, source)) {
+  JS::AutoStableStringChars linearChars(cx);
+  if (!linearChars.initTwoByte(cx, source)) {
     return nullptr;
   }
 
-  const char16_t* chars = stableChars.twoByteRange().begin().get();
   JS::SourceText<char16_t> srcBuf;
-  if (!srcBuf.init(cx, chars, source->length(),
-                   JS::SourceOwnership::Borrowed)) {
+  if (!srcBuf.initMaybeBorrowed(cx, linearChars)) {
     return nullptr;
   }
 
@@ -349,7 +487,7 @@ JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
     return nullptr;
   }
 
-  RootedObject info(cx, CreateScriptPrivate(cx, path));
+  RootedObject info(cx, js::CreateScriptPrivate(cx, path));
   if (!info) {
     return nullptr;
   }
@@ -433,8 +571,8 @@ bool ModuleLoader::getScriptPath(JSContext* cx, HandleValue privateValue,
 }
 
 JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
-                                            HandleLinearString pathArg) {
-  RootedLinearString path(cx, pathArg);
+                                            Handle<JSLinearString*> pathArg) {
+  Rooted<JSLinearString*> path(cx, pathArg);
 
   if (IsJavaScriptURL(path)) {
     return path;
@@ -448,7 +586,7 @@ JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
   }
 
   // Remove the drive letter, if present.
-  RootedLinearString drive(cx);
+  Rooted<JSLinearString*> drive(cx);
   if (path->length() > 2 && mozilla::IsAsciiAlpha(CharAt(path, 0)) &&
       CharAt(path, 1) == u':' && CharAt(path, 2) == u'\\') {
     drive = SubString(cx, path, 0, 2);
@@ -460,7 +598,7 @@ JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
 #endif  // XP_WIN
 
   // Normalize the path by removing redundant path components.
-  Rooted<GCVector<JSLinearString*>> components(cx);
+  Rooted<GCVector<JSLinearString*>> components(cx, cx);
   size_t lastSep = 0;
   while (lastSep < path->length()) {
     int32_t i = IndexOf(path, PathSeparator, lastSep);
@@ -468,7 +606,7 @@ JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
       i = path->length();
     }
 
-    RootedLinearString part(cx, SubString(cx, path, lastSep, i));
+    Rooted<JSLinearString*> part(cx, SubString(cx, path, lastSep, i));
     if (!part) {
       return nullptr;
     }
@@ -501,7 +639,7 @@ JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
     }
   }
 
-  RootedLinearString pathSep(cx, pathSeparatorStr);
+  Rooted<JSLinearString*> pathSep(cx, pathSeparatorStr);
   RootedString normalized(cx, JoinStrings(cx, components, pathSep));
   if (!normalized) {
     return nullptr;
@@ -519,7 +657,8 @@ JSLinearString* ModuleLoader::normalizePath(JSContext* cx,
   return JS_EnsureLinearString(cx, normalized);
 }
 
-JSString* ModuleLoader::fetchSource(JSContext* cx, HandleLinearString path) {
+JSString* ModuleLoader::fetchSource(JSContext* cx,
+                                    Handle<JSLinearString*> path) {
   if (IsJavaScriptURL(path)) {
     return ExtractJavaScriptURLSource(cx, path);
   }
