@@ -3,21 +3,21 @@
 
 /*---
 esid: sec-array.prototype.every
-es5id: 15.4.4.16-5-10
-description: Array.prototype.every - Array Object can be used as thisArg
+es5id: 15.4.4.16-5-12
+description: Array.prototype.every - Boolean Object can be used as thisArg
 ---*/
 
 var accessed = false;
-var objArray = [];
+var objBoolean = new Boolean();
 
 function callbackfn(val, idx, obj) {
   accessed = true;
-  return this === objArray;
+  return this === objBoolean;
 }
 
 
 
-assert([11].every(callbackfn, objArray), '[11].every(callbackfn, objArray) !== true');
+assert([11].every(callbackfn, objBoolean), '[11].every(callbackfn, objBoolean) !== true');
 assert(accessed, 'accessed !== true');
 
 reportCompare(0, 0);
