@@ -5,24 +5,23 @@
 esid: sec-array.prototype.reduce
 description: >
     Array.prototype.reduce - This object is the Arguments object which
-    implements its own property get method (number of arguments equals
-    number of parameters)
+    implements its own property get method (number of arguments is
+    less than number of parameters)
 ---*/
 
 var testResult = false;
 
 function callbackfn(prevVal, curVal, idx, obj) {
-  if (idx === 2) {
-    testResult = (prevVal === 1);
+  if (idx === 1) {
+    testResult = (prevVal === 0);
   }
 }
 
 var func = function(a, b, c) {
-  delete arguments[0];
   Array.prototype.reduce.call(arguments, callbackfn);
 };
 
-func(0, 1, 2);
+func(0, 1);
 
 assert(testResult, 'testResult !== true');
 
