@@ -10,13 +10,13 @@ es5id: 15.3_A2_T2
 description: Checking if executing "Function.call(this, "var #x  = 1;")" fails
 ---*/
 
-//CHECK#
 try {
   Function.call(this, "var #x  = 1;");
 } catch (e) {
-  if (!(e instanceof SyntaxError)) {
-    $ERROR('#1: function body must be valid');
-  }
+  assert(
+    e instanceof SyntaxError,
+    'The result of evaluating (e instanceof SyntaxError) is expected to be true'
+  );
 }
 
 reportCompare(0, 0);
