@@ -2,13 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.11-4-24
-description: >
-    Object.isSealed returns false for all built-in objects
-    (SyntaxError)
+es5id: 15.2.3.11-4-2
+description: Object.isSealed returns false for all built-in objects (Object)
 ---*/
 
-var b = Object.isSealed(SyntaxError);
+var b = Object.isSealed(Object);
 
 assert.sameValue(b, false, 'b');
 
