@@ -10,6 +10,6 @@ description: >
 
 assert.throws(TypeError, function() {
   Object.prototype.propertyIsEnumerable.call(undefined, 'foo');
-});
+}, 'Object.prototype.propertyIsEnumerable.call(undefined, "foo") throws a TypeError exception');
 
 reportCompare(0, 0);
