@@ -3,14 +3,14 @@
 
 /*---
 info: Compound Assignment Operator calls PutValue(lref, v)
-es5id: S11.13.2_A5.7_T3
+es5id: S11.13.2_A5.4_T3
 description: >
     Evaluating LeftHandSideExpression lref returns Reference type; Reference
     base value is an environment record and environment record kind is
     object environment record. PutValue(lref, v) uses the initially
     created Reference even if the environment binding is no longer present.
     Binding in surrounding object environment record is not changed.
-    Check operator is "x >>= y".
+    Check operator is "x += y".
 flags: [noStrict]
 ---*/
 
@@ -20,21 +20,21 @@ var outerScope = {
 var innerScope = {
   get x() {
     delete this.x;
-    return 16;
+    return 2;
   }
 };
 
 with (outerScope) {
   with (innerScope) {
-    x >>= 3;
+    x += 1;
   }
 }
 
-if (innerScope.x !== 2) {
-  $ERROR('#1: innerScope.x === 2. Actual: ' + (innerScope.x));
+if (innerScope.x !== 3) {
+  throw new Test262Error('#1: innerScope.x === 3. Actual: ' + (innerScope.x));
 }
 if (outerScope.x !== 0) {
-  $ERROR('#2: outerScope.x === 0. Actual: ' + (outerScope.x));
+  throw new Test262Error('#2: outerScope.x === 0. Actual: ' + (outerScope.x));
 }
 
 reportCompare(0, 0);
