@@ -6,16 +6,22 @@
 info: |
     This test is actually testing the [[Delete]] internal method (8.12.8). Since the
     language provides no way to directly exercise [[Delete]], the tests are placed here.
-es5id: 11.4.1-4.a-9-s
+esid: sec-delete-operator-runtime-semantics-evaluation
 description: >
     delete operator throws TypeError when deleting a non-configurable
-    data property (Math.LN2) in strict mode
+    data property in strict mode
 flags: [onlyStrict]
 ---*/
 
+var o = {};
+var desc = {
+  value: 1,
+}; // all other attributes default to false
+Object.defineProperty(o, 'foo', desc);
 
+// Now, deleting o.foo should throw TypeError because [[Configurable]] on foo is false.
 assert.throws(TypeError, function() {
-    delete Math.LN2;
+  delete o.foo;
 });
 
 reportCompare(0, 0);
