@@ -12,13 +12,13 @@ var x = function () { throw "x"; };
 var y = function () { throw "y"; };
 try {
    x() && y();
-   $ERROR('#1.1: var x = function () { throw "x"; }; var y = function () { throw "y"; }; x() && y() throw "x". Actual: ' + (x() && y()));
+   throw new Test262Error('#1.1: var x = function () { throw "x"; }; var y = function () { throw "y"; }; x() && y() throw "x". Actual: ' + (x() && y()));
 } catch (e) {
    if (e === "y") {
-     $ERROR('#1.2: First expression is evaluated first, and then second expression');
+     throw new Test262Error('#1.2: First expression is evaluated first, and then second expression');
    } else {
      if (e !== "x") {
-       $ERROR('#1.3: var x = function () { throw "x"; }; var y = function () { throw "y"; }; x() && y() throw "x". Actual: ' + (e));
+       throw new Test262Error('#1.3: var x = function () { throw "x"; }; var y = function () { throw "y"; }; x() && y() throw "x". Actual: ' + (e));
      }
    }
 }
