@@ -10,11 +10,11 @@ description: If GetBase(x) is null, throw ReferenceError
 //CHECK#1
 try {
   !x;
-  $ERROR('#1.1: !x throw ReferenceError. Actual: ' + (!x));  
+  throw new Test262Error('#1.1: !x throw ReferenceError. Actual: ' + (!x));  
 }
 catch (e) {
   if ((e instanceof ReferenceError) !== true) {
-    $ERROR('#1.2: !x throw ReferenceError. Actual: ' + (e));  
+    throw new Test262Error('#1.2: !x throw ReferenceError. Actual: ' + (e));  
   }
 }
 
