@@ -2,6 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
+esid: sec-identifiers-static-semantics-early-errors
 info: The "public" token can be used as identifier in non-strict code
 es5id: 7.6.1.2_A1.24ns
 description: Checking if execution of "public=1" succeeds in non-strict code
@@ -9,5 +10,12 @@ flags: [noStrict]
 ---*/
 
 var public = 1;
+var publi\u0063 = 2;
+
+{ let public = 3; }
+{ let publi\u0063 = 4; }
+
+{ const public = 5; }
+{ const publi\u0063 = 6; }
 
 reportCompare(0, 0);
