@@ -10,7 +10,7 @@ features: [generators]
 
 function* values() {
   yield 1;
-  $ERROR('This code is unreachable (following `yield` statement).');
+  throw new Test262Error('This code is unreachable (following `yield` statement).');
 }
 var iterator = values();
 var i = 0;
@@ -21,15 +21,15 @@ var result = (function() {
       i++;
       return 34;
 
-      $ERROR('This code is unreachable (following `return` statement).');
+      throw new Test262Error('This code is unreachable (following `return` statement).');
     } catch(err) {
-      $ERROR('This code is unreachable (within `catch` block).');
+      throw new Test262Error('This code is unreachable (within `catch` block).');
     }
 
-    $ERROR('This code is unreachable (following `try` statement).');
+    throw new Test262Error('This code is unreachable (following `try` statement).');
   }
 
-  $ERROR('This code is unreachable (following `for..in` statement).');
+  throw new Test262Error('This code is unreachable (following `for..in` statement).');
 })();
 
 assert.sameValue(result, 34);
