@@ -6,17 +6,18 @@ info: |
     Refer 13.1; 
     It is a SyntaxError if any Identifier value occurs more than once within a FormalParameterList of a strict mode
     FunctionDeclaration or FunctionExpression.
-es5id: 13.1-27-s
+es5id: 13.1-31-s
 description: >
     Strict Mode - SyntaxError is thrown if a function is created using
-    a FunctionDeclaration that is contained in eval strict code and
-    the function has three identical parameters
+    a FunctionExpression that is contained in eval strict code and the
+    function has two identical parameters, which are separated by a
+    unique parameter name
 flags: [noStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-            eval("'use strict'; function _13_1_27_fun(param, param, param) { }");
+            eval("'use strict'; var _13_1_31_fun = function (param1, param2, param1) { };");
 });
 
 reportCompare(0, 0);
