@@ -20,10 +20,10 @@ function __func(){
 
 try{
     var x=__func()
-    $ERROR('#0: var x=__func() lead to throwing exception');
+    throw new Test262Error('#0: var x=__func() lead to throwing exception');
 } catch(e){
     if (e !== "Catch Me If You Can") {
-    	$ERROR('#1: Exception === "Catch Me If You Can". Actual: exception ==='+e);
+    	throw new Test262Error('#1: Exception === "Catch Me If You Can". Actual: exception ==='+e);
     }
 }
 
