@@ -10,7 +10,7 @@ description: Using embedded "if" into "if" constructions
 //CHECK# 1
 if(true)
   if (false)
-    $ERROR('#1.1: At embedded "if/else" constructions engine must select right branches');
+    throw new Test262Error('#1.1: At embedded "if/else" constructions engine must select right branches');
 
 //CHECK# 2
 var c=0;
@@ -18,16 +18,16 @@ if(true)
   if (true)
     c=2;
 if (c!==2)
-    $ERROR('#2: At embedded "if/else" constructions engine must select right branches');
+    throw new Test262Error('#2: At embedded "if/else" constructions engine must select right branches');
 
 //CHECK# 3
 if(false)
   if (true)
-    $ERROR('#3.1: At embedded "if/else" constructions engine must select right branches');
+    throw new Test262Error('#3.1: At embedded "if/else" constructions engine must select right branches');
 
 //CHECK# 4
 if(false)
   if (true)
-    $ERROR('#4.1: At embedded "if/else" constructions engine must select right branches');
+    throw new Test262Error('#4.1: At embedded "if/else" constructions engine must select right branches');
 
 reportCompare(0, 0);
