@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2021 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -18,19 +18,12 @@
 #ifndef INCLUDED_JSI_CONFIGDB
 #define INCLUDED_JSI_CONFIGDB
 
-#include "ps/ConfigDB.h"
-
-class ScriptInterface;
+extern bool g_PauseOnFocusLoss;
+class ScriptRequest;
 
 namespace JSI_ConfigDB
 {
-	bool GetConfigNamespace(std::wstring cfgNsString, EConfigNamespace& cfgNs);
-	std::string GetValue(void* cbdata, std::wstring cfgNsString, std::string name);
-	bool CreateValue(void* cbdata, std::wstring cfgNsString, std::string name, std::string value);
-	bool WriteFile(void* cbdata, std::wstring cfgNsString, Path path);
-	bool Reload(void* cbdata, std::wstring cfgNsString);
-	bool SetFile(void* cbdata, std::wstring cfgNsString, Path path);
-	void RegisterScriptFunctions(ScriptInterface& scriptInterface);
+	void RegisterScriptFunctions(const ScriptRequest& rq);
 }
 
-#endif
+#endif // INCLUDED_JSI_CONFIGDB
