@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Wildfire Games.
+/* Copyright (C) 2021 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -17,18 +17,28 @@
 
 #include "lib/self_test.h"
 
-// usually defined by main.cpp, used by engine's scripting/ScriptGlue.cpp,
+// usually defined by main.cpp, used by engine's scripting/ScriptFunctions.cpp,
 // must be included here to placate linker.
-void kill_mainloop()
+
+bool IsQuitRequested()
+{
+	return false;
+}
+
+void QuitEngine()
 {
 }
 
-void restart_mainloop_in_atlas()
+void StartAtlas()
+{
+}
+
+void RestartEngine()
 {
 }
 
 // just so that cxxtestgen doesn't complain "No tests defined"
-class TestDummy : public CxxTest::TestSuite 
+class TestDummy : public CxxTest::TestSuite
 {
 public:
 	void test_dummy()
