@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2024 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,14 +19,6 @@
 
 #include "Simulation2.h"
 
-#include "simulation2/MessageTypes.h"
-#include "simulation2/system/ComponentManager.h"
-#include "simulation2/system/ParamNode.h"
-#include "simulation2/system/SimContext.h"
-#include "simulation2/components/ICmpAIManager.h"
-#include "simulation2/components/ICmpCommandQueue.h"
-#include "simulation2/components/ICmpTemplateManager.h"
-
 #include "graphics/MapReader.h"
 #include "graphics/Terrain.h"
 #include "lib/timer.h"
@@ -38,33 +30,35 @@
 #include "ps/Loader.h"
 #include "ps/Profile.h"
 #include "ps/Pyrogenesis.h"
+#include "ps/Util.h"
 #include "ps/XML/Xeromyces.h"
+#include "scriptinterface/FunctionWrapper.h"
+#include "scriptinterface/JSON.h"
+#include "scriptinterface/Object.h"
+#include "scriptinterface/ScriptContext.h"
+#include "scriptinterface/ScriptInterface.h"
+#include "scriptinterface/StructuredClone.h"
+#include "simulation2/MessageTypes.h"
+#include "simulation2/system/ComponentManager.h"
+#include "simulation2/system/ParamNode.h"
+#include "simulation2/system/SimContext.h"
+#include "simulation2/components/ICmpAIManager.h"
+#include "simulation2/components/ICmpCommandQueue.h"
+#include "simulation2/components/ICmpTemplateManager.h"
 
+#include <fstream>
 #include <iomanip>
-
-#if MSC_VERSION
-#include <process.h>
-#define getpid _getpid // use the non-deprecated function name
-#endif
-
-static std::string Hexify(const std::string& s) // TODO: shouldn't duplicate this function in so many places
-{
-	std::stringstream str;
-	str << std::hex;
-	for (size_t i = 0; i < s.size(); ++i)
-		str << std::setfill('0') << std::setw(2) << (int)(unsigned char)s[i];
-	return str.str();
-}
+#include <memory>
 
 class CSimulation2Impl
 {
 public:
-	CSimulation2Impl(CUnitManager* unitManager, CTerrain* terrain) :
-		m_SimContext(), m_ComponentManager(m_SimContext),
-		m_EnableOOSLog(false), m_EnableSerializationTest(false)
+	CSimulation2Impl(CUnitManager* unitManager, ScriptContext& cx, CTerrain* terrain) :
+		m_SimContext{terrain, unitManager},
+		m_ComponentManager{m_SimContext, cx},
+		m_MapSettings{cx.GetGeneralJSContext()},
+		m_InitAttributes{cx.GetGeneralJSContext()}
 	{
-		m_SimContext.m_UnitManager = unitManager;
-		m_SimContext.m_Terrain = terrain;
 		m_ComponentManager.LoadComponentTypes();
 
 		RegisterFileReloadFunc(ReloadChangedFileCB, this);
@@ -72,8 +66,17 @@ public:
 		// Tests won't have config initialised
 		if (CConfigDB::IsInitialised())
 		{
-			CFG_GET_VAL("ooslog", Bool, m_EnableOOSLog);
-			CFG_GET_VAL("serializationtest", Bool, m_EnableSerializationTest);
+			CFG_GET_VAL("ooslog", m_EnableOOSLog);
+			CFG_GET_VAL("serializationtest", m_EnableSerializationTest);
+			CFG_GET_VAL("rejointest", m_RejoinTestTurn);
+			if (m_RejoinTestTurn < 0) // Handle bogus values of the arg
+				m_RejoinTestTurn = -1;
+		}
+
+		if (m_EnableOOSLog)
+		{
+			m_OOSLogPath = createDateIndexSubdirectory(psLogDir() / "oos_logs");
+			debug_printf("Writing ooslogs to %s\n", m_OOSLogPath.string8().c_str());
 		}
 	}
 
@@ -93,60 +96,13 @@ public:
 	static void ResetComponentState(CComponentManager& componentManager, bool skipScriptedComponents, bool skipAI)
 	{
 		componentManager.ResetState();
-
-		CParamNode noParam;
-		CComponentManager::ComponentTypeId cid;
-
 		componentManager.InitSystemEntity();
-		CEntityHandle systemEntity = componentManager.GetSystemEntity();
-
-		// Add native system components:
-		componentManager.AddComponent(systemEntity, CID_TemplateManager, noParam);
-
-		componentManager.AddComponent(systemEntity, CID_CommandQueue, noParam);
-		componentManager.AddComponent(systemEntity, CID_ObstructionManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_ParticleManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_Pathfinder, noParam);
-		componentManager.AddComponent(systemEntity, CID_ProjectileManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_RangeManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_SoundManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_Terrain, noParam);
-		componentManager.AddComponent(systemEntity, CID_TerritoryManager, noParam);
-		componentManager.AddComponent(systemEntity, CID_WaterManager, noParam);
-
-		// Add scripted system components:
-		if (!skipScriptedComponents)
-		{
-			// TODO: Load this from a file to allow modders to add scripted components
-			// without having to recompile.
-#define LOAD_SCRIPTED_COMPONENT(name) \
-			cid = componentManager.LookupCID(name); \
-			if (cid == CID__Invalid) \
-				LOGERROR(L"Can't find component type " L##name); \
-			componentManager.AddComponent(systemEntity, cid, noParam)
-
-			LOAD_SCRIPTED_COMPONENT("AIInterface");
-			LOAD_SCRIPTED_COMPONENT("AuraManager");
-			LOAD_SCRIPTED_COMPONENT("Barter");
-			LOAD_SCRIPTED_COMPONENT("EndGameManager");
-			LOAD_SCRIPTED_COMPONENT("GuiInterface");
-			LOAD_SCRIPTED_COMPONENT("PlayerManager");
-			LOAD_SCRIPTED_COMPONENT("TechnologyTemplateManager");
-			LOAD_SCRIPTED_COMPONENT("Timer");
-			LOAD_SCRIPTED_COMPONENT("ValueModificationManager");
-
-#undef LOAD_SCRIPTED_COMPONENT
-
-			if (!skipAI)
-			{
-				componentManager.AddComponent(systemEntity, CID_AIManager, noParam);
-			}
-		
-		}
+		componentManager.AddSystemComponents(skipScriptedComponents, skipAI);
 	}
 
 	static bool LoadDefaultScripts(CComponentManager& componentManager, std::set<VfsPath>* loadedScripts);
 	static bool LoadScripts(CComponentManager& componentManager, std::set<VfsPath>* loadedScripts, const VfsPath& path);
+	static bool LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings, std::set<VfsPath>* loadedScripts);
 	Status ReloadChangedFile(const VfsPath& path);
 
 	static Status ReloadChangedFileCB(void* param, const VfsPath& path)
@@ -167,19 +123,27 @@ public:
 	float m_LastFrameOffset;
 
 	std::string m_StartupScript;
-	CScriptValRooted m_InitAttributes;
-	CScriptValRooted m_MapSettings;
+	JS::PersistentRootedValue m_InitAttributes;
+	JS::PersistentRootedValue m_MapSettings;
 
 	std::set<VfsPath> m_LoadedScripts;
 
 	uint32_t m_TurnNumber;
 
-	bool m_EnableOOSLog;
-
+	bool m_EnableOOSLog{false};
+	OsPath m_OOSLogPath;
 
 	// Functions and data for the serialization test mode: (see Update() for relevant comments)
 
-	bool m_EnableSerializationTest;
+	bool m_EnableSerializationTest{false};
+	int m_RejoinTestTurn{-1};
+	bool m_TestingRejoin{false};
+
+	// Secondary simulation (NB: order matters for destruction).
+	std::unique_ptr<CComponentManager> m_SecondaryComponentManager;
+	std::unique_ptr<CTerrain> m_SecondaryTerrain;
+	std::unique_ptr<CSimContext> m_SecondaryContext;
+	std::unique_ptr<std::set<VfsPath>> m_SecondaryLoadedScripts;
 
 	struct SerializationTestState
 	{
@@ -194,14 +158,22 @@ public:
 		SerializationTestState* primaryStateBefore, SerializationTestState* primaryStateAfter,
 		SerializationTestState* secondaryStateBefore, SerializationTestState* secondaryStateAfter);
 
-	static std::vector<SimulationCommand> CloneCommandsFromOtherContext(ScriptInterface& oldScript, ScriptInterface& newScript,
+	void InitRNGSeedSimulation();
+	void InitRNGSeedAI();
+
+	static std::vector<SimulationCommand> CloneCommandsFromOtherCompartment(const ScriptInterface& newScript, const ScriptInterface& oldScript,
 		const std::vector<SimulationCommand>& commands)
 	{
-		std::vector<SimulationCommand> newCommands = commands;
-		for (size_t i = 0; i < newCommands.size(); ++i)
+		std::vector<SimulationCommand> newCommands;
+		newCommands.reserve(commands.size());
+
+		ScriptRequest rqNew(newScript);
+		for (const SimulationCommand& command : commands)
 		{
-			newCommands[i].data = CScriptValRooted(newScript.GetContext(),
-				newScript.CloneValueFromOtherContext(oldScript, newCommands[i].data.get()));
+			JS::RootedValue tmpCommand(rqNew.cx, Script::CloneValueFromOtherCompartment(newScript, oldScript, command.data));
+			Script::DeepFreezeObject(rqNew, tmpCommand);
+			SimulationCommand cmd(command.player, rqNew.cx, tmpCommand);
+			newCommands.emplace_back(std::move(cmd));
 		}
 		return newCommands;
 	}
@@ -223,18 +195,42 @@ bool CSimulation2Impl::LoadScripts(CComponentManager& componentManager, std::set
 		return false;
 
 	bool ok = true;
-	for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
+	for (const VfsPath& scriptPath : pathnames)
 	{
-		VfsPath filename = *it;
 		if (loadedScripts)
-			loadedScripts->insert(filename);
-		LOGMESSAGE(L"Loading simulation script '%ls'", filename.string().c_str());
-		if (! componentManager.LoadScript(filename))
+			loadedScripts->insert(scriptPath);
+		LOGMESSAGE("Loading simulation script '%s'", scriptPath.string8());
+		if (!componentManager.LoadScript(scriptPath))
 			ok = false;
 	}
 	return ok;
 }
 
+bool CSimulation2Impl::LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings, std::set<VfsPath>* loadedScripts)
+{
+	bool ok = true;
+	ScriptRequest rq(componentManager.GetScriptInterface());
+	if (Script::HasProperty(rq, mapSettings, "TriggerScripts"))
+	{
+		std::vector<std::string> scriptNames;
+		Script::GetProperty(rq, mapSettings, "TriggerScripts", scriptNames);
+		for (const std::string& triggerScript : scriptNames)
+		{
+			std::string scriptName = "maps/" + triggerScript;
+			if (loadedScripts)
+			{
+				if (loadedScripts->find(scriptName) != loadedScripts->end())
+					continue;
+				loadedScripts->insert(scriptName);
+			}
+			LOGMESSAGE("Loading trigger script '%s'", scriptName.c_str());
+			if (!componentManager.LoadScript(scriptName.data()))
+				ok = false;
+		}
+	}
+	return ok;
+}
+
 Status CSimulation2Impl::ReloadChangedFile(const VfsPath& path)
 {
 	// Ignore if this file wasn't loaded as a script
@@ -247,7 +243,7 @@ Status CSimulation2Impl::ReloadChangedFile(const VfsPath& path)
 	if (!VfsFileExists(path))
 		return INFO::OK;
 
-	LOGMESSAGE(L"Reloading simulation script '%ls'", path.string().c_str());
+	LOGMESSAGE("Reloading simulation script '%s'", path.string8());
 	if (!m_ComponentManager.LoadScript(path, true))
 		return ERR::FAIL;
 
@@ -286,19 +282,19 @@ void CSimulation2Impl::DumpSerializationTestState(SerializationTestState& state,
 {
 	if (!state.hash.empty())
 	{
-		std::ofstream file (OsString(path / (L"hash." + suffix)).c_str(), std::ofstream::out | std::ofstream::trunc);
+		std::ofstream file (OsString(path / (L"hash." + suffix)), std::ofstream::out | std::ofstream::trunc);
 		file << Hexify(state.hash);
 	}
 
 	if (!state.debug.str().empty())
 	{
-		std::ofstream file (OsString(path / (L"debug." + suffix)).c_str(), std::ofstream::out | std::ofstream::trunc);
+		std::ofstream file (OsString(path / (L"debug." + suffix)), std::ofstream::out | std::ofstream::trunc);
 		file << state.debug.str();
 	}
 
 	if (!state.state.str().empty())
 	{
-		std::ofstream file (OsString(path / (L"state." + suffix)).c_str(), std::ofstream::out | std::ofstream::trunc | std::ofstream::binary);
+		std::ofstream file (OsString(path / (L"state." + suffix)), std::ofstream::out | std::ofstream::trunc | std::ofstream::binary);
 		file << state.state.str();
 	}
 }
@@ -307,8 +303,8 @@ void CSimulation2Impl::ReportSerializationFailure(
 	SerializationTestState* primaryStateBefore, SerializationTestState* primaryStateAfter,
 	SerializationTestState* secondaryStateBefore, SerializationTestState* secondaryStateAfter)
 {
-	OsPath path = psLogDir() / "oos_log";
-	CreateDirectories(path, 0700);
+	const OsPath path = createDateIndexSubdirectory(psLogDir() / "serializationtest");
+	debug_printf("Writing serializationtest-data to %s\n", path.string8().c_str());
 
 	// Clean up obsolete files from previous runs
 	wunlink(path / "hash.before.a");
@@ -336,6 +332,30 @@ void CSimulation2Impl::ReportSerializationFailure(
 	debug_warn(L"Serialization test failure");
 }
 
+void CSimulation2Impl::InitRNGSeedSimulation()
+{
+	u32 seed = 0;
+	ScriptRequest rq(m_ComponentManager.GetScriptInterface());
+	if (!Script::HasProperty(rq, m_MapSettings, "Seed") ||
+		!Script::GetProperty(rq, m_MapSettings, "Seed", seed))
+		LOGWARNING("CSimulation2Impl::InitRNGSeedSimulation: No seed value specified - using %d", seed);
+
+	m_ComponentManager.SetRNGSeed(seed);
+}
+
+void CSimulation2Impl::InitRNGSeedAI()
+{
+	u32 seed = 0;
+	ScriptRequest rq(m_ComponentManager.GetScriptInterface());
+	if (!Script::HasProperty(rq, m_MapSettings, "AISeed") ||
+		!Script::GetProperty(rq, m_MapSettings, "AISeed", seed))
+		LOGWARNING("CSimulation2Impl::InitRNGSeedAI: No seed value specified - using %d", seed);
+
+	CmpPtr<ICmpAIManager> cmpAIManager(m_SimContext, SYSTEM_ENTITY);
+	if (cmpAIManager)
+		cmpAIManager->SetRNGSeed(seed);
+}
+
 void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationCommand>& commands)
 {
 	PROFILE3("sim update");
@@ -351,13 +371,23 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 	 * Then we run the update on the secondary context, and check that its new serialized
 	 * state matches the primary context after the update (to check that the simulation doesn't depend
 	 * on anything that's not serialized).
+	 *
+	 * In rejoin test mode, the secondary simulation is initialized from serialized data at turn N, then both
+	 * simulations run independantly while comparing their states each turn. This is way faster than a
+	 * complete serialization test and allows us to reproduce OOSes on rejoin.
 	 */
 
 	const bool serializationTestDebugDump = false; // set true to save human-readable state dumps before an error is detected, for debugging (but slow)
 	const bool serializationTestHash = true; // set true to save and compare hash of state
 
 	SerializationTestState primaryStateBefore;
-	if (m_EnableSerializationTest)
+	const ScriptInterface& scriptInterface = m_ComponentManager.GetScriptInterface();
+
+	const bool startRejoinTest = (int64_t) m_RejoinTestTurn == m_TurnNumber;
+	if (startRejoinTest)
+		m_TestingRejoin = true;
+
+	if (m_EnableSerializationTest || m_TestingRejoin)
 	{
 		ENSURE(m_ComponentManager.SerializeState(primaryStateBefore.state));
 		if (serializationTestDebugDump)
@@ -366,54 +396,69 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 			ENSURE(m_ComponentManager.ComputeStateHash(primaryStateBefore.hash, false));
 	}
 
-
 	UpdateComponents(m_SimContext, turnLengthFixed, commands);
 
-
-	if (m_EnableSerializationTest)
+	if (m_EnableSerializationTest || startRejoinTest)
 	{
-		// Initialise the secondary simulation
-		CTerrain secondaryTerrain;
-		CSimContext secondaryContext;
-		secondaryContext.m_Terrain = &secondaryTerrain;
-		CComponentManager secondaryComponentManager(secondaryContext);
-		secondaryComponentManager.LoadComponentTypes();
-		ENSURE(LoadDefaultScripts(secondaryComponentManager, NULL));
-		ResetComponentState(secondaryComponentManager, false, false);
+		if (startRejoinTest)
+			debug_printf("Initializing the secondary simulation\n");
+
+		m_SecondaryTerrain = std::make_unique<CTerrain>();
+
+		m_SecondaryContext = std::make_unique<CSimContext>(m_SecondaryTerrain.get());
+
+		m_SecondaryComponentManager = std::make_unique<CComponentManager>(*m_SecondaryContext, scriptInterface.GetContext());
+		m_SecondaryComponentManager->LoadComponentTypes();
+
+		m_SecondaryLoadedScripts = std::make_unique<std::set<VfsPath>>();
+		ENSURE(LoadDefaultScripts(*m_SecondaryComponentManager, m_SecondaryLoadedScripts.get()));
+		ResetComponentState(*m_SecondaryComponentManager, false, false);
+
+		ScriptRequest rq(scriptInterface);
+
+		// Load the trigger scripts after we have loaded the simulation.
+		{
+			ScriptRequest rq2(m_SecondaryComponentManager->GetScriptInterface());
+			JS::RootedValue mapSettingsCloned(rq2.cx, Script::CloneValueFromOtherCompartment(m_SecondaryComponentManager->GetScriptInterface(), scriptInterface, m_MapSettings));
+			ENSURE(LoadTriggerScripts(*m_SecondaryComponentManager, mapSettingsCloned, m_SecondaryLoadedScripts.get()));
+		}
 
 		// Load the map into the secondary simulation
 
 		LDR_BeginRegistering();
-		CMapReader* mapReader = new CMapReader; // automatically deletes itself
+		std::unique_ptr<CMapReader> mapReader = std::make_unique<CMapReader>();
 
-		// TODO: this duplicates CWorld::RegisterInit and could probably be cleaned up a bit
 		std::string mapType;
-		m_ComponentManager.GetScriptInterface().GetProperty(m_InitAttributes.get(), "mapType", mapType);
+		Script::GetProperty(rq, m_InitAttributes, "mapType", mapType);
 		if (mapType == "random")
 		{
 			// TODO: support random map scripts
-			debug_warn(L"Serialization test mode only supports scenarios");
+			debug_warn(L"Serialization test mode does not support random maps");
 		}
 		else
 		{
 			std::wstring mapFile;
-			m_ComponentManager.GetScriptInterface().GetProperty(m_InitAttributes.get(), "map", mapFile);
+			Script::GetProperty(rq, m_InitAttributes, "map", mapFile);
 
 			VfsPath mapfilename = VfsPath(mapFile).ChangeExtension(L".pmp");
-			mapReader->LoadMap(mapfilename, CScriptValRooted(), &secondaryTerrain, NULL, NULL, NULL, NULL, NULL, NULL,
-				NULL, NULL, &secondaryContext, INVALID_PLAYER, true); // throws exception on failure
+			mapReader->LoadMap(mapfilename, scriptInterface.GetContext(), JS::UndefinedHandleValue,
+				m_SecondaryTerrain.get(), NULL, NULL, NULL, NULL, NULL, NULL,
+				NULL, NULL, m_SecondaryContext.get(), INVALID_PLAYER, true); // throws exception on failure
 		}
+
 		LDR_EndRegistering();
 		ENSURE(LDR_NonprogressiveLoad() == INFO::OK);
+		ENSURE(m_SecondaryComponentManager->DeserializeState(primaryStateBefore.state));
+	}
 
-		ENSURE(secondaryComponentManager.DeserializeState(primaryStateBefore.state));
-
+	if (m_EnableSerializationTest || m_TestingRejoin)
+	{
 		SerializationTestState secondaryStateBefore;
-		ENSURE(secondaryComponentManager.SerializeState(secondaryStateBefore.state));
+		ENSURE(m_SecondaryComponentManager->SerializeState(secondaryStateBefore.state));
 		if (serializationTestDebugDump)
-			ENSURE(secondaryComponentManager.DumpDebugState(secondaryStateBefore.debug, false));
+			ENSURE(m_SecondaryComponentManager->DumpDebugState(secondaryStateBefore.debug, false));
 		if (serializationTestHash)
-			ENSURE(secondaryComponentManager.ComputeStateHash(secondaryStateBefore.hash, false));
+			ENSURE(m_SecondaryComponentManager->ComputeStateHash(secondaryStateBefore.hash, false));
 
 		if (primaryStateBefore.state.str() != secondaryStateBefore.state.str() ||
 			primaryStateBefore.hash != secondaryStateBefore.hash)
@@ -426,42 +471,42 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 		if (serializationTestHash)
 			ENSURE(m_ComponentManager.ComputeStateHash(primaryStateAfter.hash, false));
 
-		UpdateComponents(secondaryContext, turnLengthFixed,
-			CloneCommandsFromOtherContext(m_ComponentManager.GetScriptInterface(), secondaryComponentManager.GetScriptInterface(), commands));
-
+		UpdateComponents(*m_SecondaryContext, turnLengthFixed,
+			CloneCommandsFromOtherCompartment(m_SecondaryComponentManager->GetScriptInterface(), scriptInterface, commands));
 		SerializationTestState secondaryStateAfter;
-		ENSURE(secondaryComponentManager.SerializeState(secondaryStateAfter.state));
+		ENSURE(m_SecondaryComponentManager->SerializeState(secondaryStateAfter.state));
 		if (serializationTestHash)
-			ENSURE(secondaryComponentManager.ComputeStateHash(secondaryStateAfter.hash, false));
+			ENSURE(m_SecondaryComponentManager->ComputeStateHash(secondaryStateAfter.hash, false));
 
 		if (primaryStateAfter.state.str() != secondaryStateAfter.state.str() ||
 			primaryStateAfter.hash != secondaryStateAfter.hash)
 		{
 			// Only do the (slow) dumping now we know we're going to need to report it
 			ENSURE(m_ComponentManager.DumpDebugState(primaryStateAfter.debug, false));
-			ENSURE(secondaryComponentManager.DumpDebugState(secondaryStateAfter.debug, false));
+			ENSURE(m_SecondaryComponentManager->DumpDebugState(secondaryStateAfter.debug, false));
 
 			ReportSerializationFailure(&primaryStateBefore, &primaryStateAfter, &secondaryStateBefore, &secondaryStateAfter);
 		}
 	}
 
-//	if (m_TurnNumber == 0)
-//		m_ComponentManager.GetScriptInterface().DumpHeap();
-
 	// Run the GC occasionally
+	// No delay because a lot of garbage accumulates in one turn and in non-visual replays there are
+	// much more turns in the same time than in normal games.
+	// Every 500 turns we run a shrinking GC, which decommits unused memory and frees all JIT code.
+	// Based on testing, this seems to be a good compromise between memory usage and performance.
+	// Also check the comment about gcPreserveCode in the ScriptInterface code and this forum topic:
+	// http://www.wildfiregames.com/forum/index.php?showtopic=18466&p=300323
+	//
 	// (TODO: we ought to schedule this for a frame where we're not
 	// running the sim update, to spread the load)
-	if (m_TurnNumber % 10 == 0)
-		m_ComponentManager.GetScriptInterface().MaybeGC();
+	if (m_TurnNumber % 500 == 0)
+		scriptInterface.GetContext().ShrinkingGC();
+	else
+		scriptInterface.GetContext().MaybeIncrementalGC(0.0f);
 
 	if (m_EnableOOSLog)
 		DumpState();
 
-	// Start computing AI for the next turn
-	CmpPtr<ICmpAIManager> cmpAIManager(m_SimContext, SYSTEM_ENTITY);
-	if (cmpAIManager)
-		cmpAIManager->StartComputation();
-
 	++m_TurnNumber;
 }
 
@@ -472,17 +517,16 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	CComponentManager& componentManager = simContext.GetComponentManager();
 
-	CMessageTurnStart msgTurnStart;
-	componentManager.BroadcastMessage(msgTurnStart);
-
 	CmpPtr<ICmpPathfinder> cmpPathfinder(simContext, SYSTEM_ENTITY);
 	if (cmpPathfinder)
-		cmpPathfinder->FinishAsyncRequests();
+		cmpPathfinder->SendRequestedPaths();
+
+	{
+		PROFILE2("Sim - Update Start");
+		CMessageTurnStart msgTurnStart;
+		componentManager.BroadcastMessage(msgTurnStart);
+	}
 
-	// Push AI commands onto the queue before we use them
-	CmpPtr<ICmpAIManager> cmpAIManager(simContext, SYSTEM_ENTITY);
-	if (cmpAIManager)
-		cmpAIManager->PushCommands();
 
 	CmpPtr<ICmpCommandQueue> cmpCommandQueue(simContext, SYSTEM_ENTITY);
 	if (cmpCommandQueue)
@@ -490,13 +534,17 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	// Process newly generated move commands so the UI feels snappy
 	if (cmpPathfinder)
-		cmpPathfinder->ProcessSameTurnMoves();
-
+	{
+		cmpPathfinder->StartProcessingMoves(true);
+		cmpPathfinder->SendRequestedPaths();
+	}
 	// Send all the update phases
 	{
+		PROFILE2("Sim - Update");
 		CMessageUpdate msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
+
 	{
 		CMessageUpdate_MotionFormation msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
@@ -504,23 +552,41 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	// Process move commands for formations (group proxy)
 	if (cmpPathfinder)
-		cmpPathfinder->ProcessSameTurnMoves();
+	{
+		cmpPathfinder->StartProcessingMoves(true);
+		cmpPathfinder->SendRequestedPaths();
+	}
 
 	{
+		PROFILE2("Sim - Motion Unit");
 		CMessageUpdate_MotionUnit msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
 	{
+		PROFILE2("Sim - Update Final");
 		CMessageUpdate_Final msgUpdate(turnLengthFixed);
 		componentManager.BroadcastMessage(msgUpdate);
 	}
 
-	// Process moves resulting from group proxy movement (unit needs to catch up or realign) and any others
-	if (cmpPathfinder)
-		cmpPathfinder->ProcessSameTurnMoves();
-
 	// Clean up any entities destroyed during the simulation update
 	componentManager.FlushDestroyedComponents();
+
+	// Compute AI immediately at turn's end.
+	CmpPtr<ICmpAIManager> cmpAIManager(simContext, SYSTEM_ENTITY);
+	if (cmpAIManager)
+	{
+		cmpAIManager->StartComputation();
+		cmpAIManager->PushCommands();
+	}
+
+	// Process all remaining moves
+	if (cmpPathfinder)
+	{
+		cmpPathfinder->UpdateGrid();
+		cmpPathfinder->StartProcessingMoves(false);
+	}
+
+	componentManager.GetScriptInterface().GetContext().RunJobs();
 }
 
 void CSimulation2Impl::Interpolate(float simFrameLength, float frameOffset, float realFrameLength)
@@ -540,13 +606,16 @@ void CSimulation2Impl::DumpState()
 {
 	PROFILE("DumpState");
 
-	std::stringstream pid;
-	pid << getpid();
 	std::stringstream name;\
 	name << std::setw(5) << std::setfill('0') << m_TurnNumber << ".txt";
-	OsPath path = psLogDir() / "sim_log" / pid.str() / name.str();
-	CreateDirectories(path.Parent(), 0700);
-	std::ofstream file (OsString(path).c_str(), std::ofstream::out | std::ofstream::trunc);
+	const OsPath path = m_OOSLogPath / name.str();
+	std::ofstream file (OsString(path), std::ofstream::out | std::ofstream::trunc);
+
+	if (!DirectoryExists(m_OOSLogPath))
+	{
+		LOGWARNING("OOS-log directory %s was deleted, creating it again.", m_OOSLogPath.string8().c_str());
+		CreateDirectories(m_OOSLogPath, 0700);
+	}
 
 	file << "State hash: " << std::hex;
 	std::string hashRaw;
@@ -559,14 +628,14 @@ void CSimulation2Impl::DumpState()
 
 	m_ComponentManager.DumpDebugState(file, true);
 
-	std::ofstream binfile (OsString(path.ChangeExtension(L".dat")).c_str(), std::ofstream::out | std::ofstream::trunc | std::ofstream::binary);
+	std::ofstream binfile (OsString(path.ChangeExtension(L".dat")), std::ofstream::out | std::ofstream::trunc | std::ofstream::binary);
 	m_ComponentManager.SerializeState(binfile);
 }
 
 ////////////////////////////////////////////////////////////////
 
-CSimulation2::CSimulation2(CUnitManager* unitManager, CTerrain* terrain) :
-	m(new CSimulation2Impl(unitManager, terrain))
+CSimulation2::CSimulation2(CUnitManager* unitManager, ScriptContext& cx, CTerrain* terrain) :
+	m(new CSimulation2Impl(unitManager, cx, terrain))
 {
 }
 
@@ -577,16 +646,27 @@ CSimulation2::~CSimulation2()
 
 // Forward all method calls to the appropriate CSimulation2Impl/CComponentManager methods:
 
-void CSimulation2::EnableOOSLog()
-{
-	m->m_EnableOOSLog = true;
-}
-
 void CSimulation2::EnableSerializationTest()
 {
 	m->m_EnableSerializationTest = true;
 }
 
+void CSimulation2::EnableRejoinTest(int rejoinTestTurn)
+{
+	m->m_RejoinTestTurn = rejoinTestTurn;
+}
+
+void CSimulation2::EnableOOSLog()
+{
+	if (m->m_EnableOOSLog)
+		return;
+
+	m->m_EnableOOSLog = true;
+	m->m_OOSLogPath = createDateIndexSubdirectory(psLogDir() / "oos_logs");
+
+	debug_printf("Writing ooslogs to %s\n", m->m_OOSLogPath.string8().c_str());
+}
+
 entity_id_t CSimulation2::AddEntity(const std::wstring& templateName)
 {
 	return m->m_ComponentManager.AddEntity(templateName, m->m_ComponentManager.AllocateNewEntity());
@@ -647,14 +727,23 @@ ScriptInterface& CSimulation2::GetScriptInterface() const
 	return m->m_ComponentManager.GetScriptInterface();
 }
 
-void CSimulation2::ReplaceSkirmishGlobals()
+void CSimulation2::PreInitGame()
 {
-	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "ReplaceSkirmishGlobals");
+	ScriptRequest rq(GetScriptInterface());
+	JS::RootedValue global(rq.cx, rq.globalValue());
+	ScriptFunction::CallVoid(rq, global, "PreInitGame");
 }
 
-void CSimulation2::InitGame(const CScriptVal& data)
+void CSimulation2::InitGame()
 {
-	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "InitGame", data);
+	ScriptRequest rq(GetScriptInterface());
+	JS::RootedValue global(rq.cx, rq.globalValue());
+
+	JS::RootedValue settings(rq.cx);
+	JS::RootedValue tmpInitAttributes(rq.cx, GetInitAttributes());
+	Script::GetProperty(rq, tmpInitAttributes, "settings", &settings);
+
+	ScriptFunction::CallVoid(rq, global, "InitGame", settings);
 }
 
 void CSimulation2::Update(int turnLength)
@@ -706,48 +795,68 @@ const std::string& CSimulation2::GetStartupScript()
 	return m->m_StartupScript;
 }
 
-void CSimulation2::SetInitAttributes(const CScriptValRooted& attribs)
+void CSimulation2::SetInitAttributes(JS::HandleValue attribs)
 {
 	m->m_InitAttributes = attribs;
 }
 
-CScriptValRooted CSimulation2::GetInitAttributes()
+JS::Value CSimulation2::GetInitAttributes()
 {
-	return m->m_InitAttributes;
+	return m->m_InitAttributes.get();
+}
+
+void CSimulation2::GetInitAttributes(JS::MutableHandleValue ret)
+{
+	ret.set(m->m_InitAttributes);
 }
 
 void CSimulation2::SetMapSettings(const std::string& settings)
 {
-	m->m_MapSettings = m->m_ComponentManager.GetScriptInterface().ParseJSON(settings);
+	Script::ParseJSON(ScriptRequest(m->m_ComponentManager.GetScriptInterface()), settings, &m->m_MapSettings);
 }
 
-void CSimulation2::SetMapSettings(const CScriptValRooted& settings)
+void CSimulation2::SetMapSettings(JS::HandleValue settings)
 {
 	m->m_MapSettings = settings;
+
+	m->InitRNGSeedSimulation();
+	m->InitRNGSeedAI();
 }
 
 std::string CSimulation2::GetMapSettingsString()
 {
-	return m->m_ComponentManager.GetScriptInterface().StringifyJSON(m->m_MapSettings.get());
+	return Script::StringifyJSON(ScriptRequest(m->m_ComponentManager.GetScriptInterface()), &m->m_MapSettings);
 }
 
-CScriptVal CSimulation2::GetMapSettings()
+void CSimulation2::GetMapSettings(JS::MutableHandleValue ret)
 {
-	return m->m_MapSettings.get();
+	ret.set(m->m_MapSettings);
 }
 
 void CSimulation2::LoadPlayerSettings(bool newPlayers)
 {
-	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "LoadPlayerSettings", m->m_MapSettings, newPlayers);
+	ScriptRequest rq(GetScriptInterface());
+	JS::RootedValue global(rq.cx, rq.globalValue());
+	ScriptFunction::CallVoid(rq, global, "LoadPlayerSettings", m->m_MapSettings, newPlayers);
 }
 
 void CSimulation2::LoadMapSettings()
 {
+	ScriptRequest rq(GetScriptInterface());
+
+	JS::RootedValue global(rq.cx, rq.globalValue());
+
 	// Initialize here instead of in Update()
-	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "LoadMapSettings", m->m_MapSettings);
+	ScriptFunction::CallVoid(rq, global, "LoadMapSettings", m->m_MapSettings);
+
+	Script::DeepFreezeObject(rq, m->m_InitAttributes);
+	GetScriptInterface().SetGlobal("InitAttributes", m->m_InitAttributes, true, true, true);
 
 	if (!m->m_StartupScript.empty())
 		GetScriptInterface().LoadScript(L"map startup script", m->m_StartupScript);
+
+	// Load the trigger scripts after we have loaded the simulation and the map.
+	m->LoadTriggerScripts(m->m_ComponentManager, m->m_MapSettings, &m->m_LoadedScripts);
 }
 
 int CSimulation2::ProgressiveLoad()
@@ -772,6 +881,7 @@ bool CSimulation2::ComputeStateHash(std::string& outHash, bool quick)
 
 bool CSimulation2::DumpDebugState(std::ostream& stream)
 {
+	stream << "sim turn: " << m->m_TurnNumber << std::endl;
 	return m->m_ComponentManager.DumpDebugState(stream, true);
 }
 
@@ -786,128 +896,100 @@ bool CSimulation2::DeserializeState(std::istream& stream)
 	return m->m_ComponentManager.DeserializeState(stream);
 }
 
+void CSimulation2::ActivateRejoinTest(int turn)
+{
+	if (m->m_RejoinTestTurn != -1)
+		return;
+	LOGMESSAGERENDER("Rejoin test will activate in %i turns", turn - m->m_TurnNumber);
+	m->m_RejoinTestTurn = turn;
+}
+
 std::string CSimulation2::GenerateSchema()
 {
 	return m->m_ComponentManager.GenerateSchema();
 }
 
-std::vector<std::string> CSimulation2::GetRMSData()
+static std::vector<std::string> GetJSONData(const VfsPath& path)
 {
-	VfsPath path(L"maps/random/");
 	VfsPaths pathnames;
-
-	std::vector<std::string> data;
-
-	// Find all ../maps/random/*.json
 	Status ret = vfs::GetPathnames(g_VFS, path, L"*.json", pathnames);
-	if (ret == INFO::OK)
-	{
-		for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
-		{
-			// Load JSON file
-			CVFSFile file;
-			PSRETURN ret = file.Load(g_VFS, *it);
-			if (ret != PSRETURN_OK)
-			{
-				LOGERROR(L"Failed to load file '%ls': %hs", path.string().c_str(), GetErrorString(ret));
-			}
-			else
-			{
-				data.push_back(file.DecodeUTF8()); // assume it's UTF-8
-			}
-		}
-	}
-	else
+	if (ret != INFO::OK)
 	{
 		// Some error reading directory
 		wchar_t error[200];
-		LOGERROR(L"Error reading directory '%ls': %ls", path.string().c_str(), StatusDescription(ret, error, ARRAY_SIZE(error)));
+		LOGERROR("Error reading directory '%s': %s", path.string8(), utf8_from_wstring(StatusDescription(ret, error, ARRAY_SIZE(error))));
+		return std::vector<std::string>();
+	}
+
+	std::vector<std::string> data;
+	for (const VfsPath& p : pathnames)
+	{
+		// Load JSON file
+		CVFSFile file;
+		PSRETURN loadStatus = file.Load(g_VFS, p);
+		if (loadStatus != PSRETURN_OK)
+		{
+			LOGERROR("GetJSONData: Failed to load file '%s': %s", p.string8(), GetErrorString(loadStatus));
+			continue;
+		}
+
+		data.push_back(file.DecodeUTF8()); // assume it's UTF-8
 	}
 
 	return data;
 }
 
-std::vector<std::string> CSimulation2::GetCivData()
+std::vector<std::string> CSimulation2::GetRMSData()
 {
-	VfsPath path(L"civs/");
-	VfsPaths pathnames;
+	return GetJSONData(L"maps/random/");
+}
 
-	std::vector<std::string> data;
+std::vector<std::string> CSimulation2::GetVictoryConditiondData()
+{
+	return GetJSONData(L"simulation/data/settings/victory_conditions/");
+}
 
-	// Load all JSON files in civs directory
-	Status ret = vfs::GetPathnames(g_VFS, path, L"*.json", pathnames);
-	if (ret == INFO::OK)
+static std::string ReadJSON(const VfsPath& path)
+{
+	if (!VfsFileExists(path))
 	{
-		for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
-		{
-			// Load JSON file
-			CVFSFile file;
-			PSRETURN ret = file.Load(g_VFS, *it);
-			if (ret != PSRETURN_OK)
-			{
-				LOGERROR(L"CSimulation2::GetCivData: Failed to load file '%ls': %hs", path.string().c_str(), GetErrorString(ret));
-			}
-			else
-			{
-				data.push_back(file.DecodeUTF8()); // assume it's UTF-8
-			}
-		}
-	}
-	else
-	{
-		// Some error reading directory
-		wchar_t error[200];
-		LOGERROR(L"CSimulation2::GetCivData: Error reading directory '%ls': %ls", path.string().c_str(), StatusDescription(ret, error, ARRAY_SIZE(error)));
+		LOGERROR("File '%s' does not exist", path.string8());
+		return std::string();
 	}
 
-	return data;
+	// Load JSON file
+	CVFSFile file;
+	PSRETURN ret = file.Load(g_VFS, path);
+	if (ret != PSRETURN_OK)
+	{
+		LOGERROR("Failed to load file '%s': %s", path.string8(), GetErrorString(ret));
+		return std::string();
+	}
+
+	return file.DecodeUTF8(); // assume it's UTF-8
 }
 
 std::string CSimulation2::GetPlayerDefaults()
 {
-	return ReadJSON(L"simulation/data/player_defaults.json");
+	return ReadJSON(L"simulation/data/settings/player_defaults.json");
 }
 
 std::string CSimulation2::GetMapSizes()
 {
-	return ReadJSON(L"simulation/data/map_sizes.json");
-}
-
-std::string CSimulation2::ReadJSON(VfsPath path)
-{
-	std::string data;
-
-	if (!VfsFileExists(path))
-	{
-		LOGERROR(L"File '%ls' does not exist", path.string().c_str());
-	}
-	else
-	{
-		// Load JSON file
-		CVFSFile file;
-		PSRETURN ret = file.Load(g_VFS, path);
-		if (ret != PSRETURN_OK)
-		{
-			LOGERROR(L"Failed to load file '%ls': %hs", path.string().c_str(), GetErrorString(ret));
-		}
-		else
-		{
-			data = file.DecodeUTF8(); // assume it's UTF-8
-		}
-	}
-
-	return data;
+	return ReadJSON(L"simulation/data/settings/map_sizes.json");
 }
 
 std::string CSimulation2::GetAIData()
 {
-	ScriptInterface& scriptInterface = GetScriptInterface();
-	std::vector<CScriptValRooted> aiData = ICmpAIManager::GetAIs(scriptInterface);
-	
+	const ScriptInterface& scriptInterface = GetScriptInterface();
+	ScriptRequest rq(scriptInterface);
+	JS::RootedValue aiData(rq.cx, ICmpAIManager::GetAIs(scriptInterface));
+
 	// Build single JSON string with array of AI data
-	CScriptValRooted ais;
-	if (!scriptInterface.Eval("({})", ais) || !scriptInterface.SetProperty(ais.get(), "AIData", aiData))
+	JS::RootedValue ais(rq.cx);
+
+	if (!Script::CreateObject(rq, &ais, "AIData", aiData))
 		return std::string();
-	
-	return scriptInterface.StringifyJSON(ais.get());
+
+	return Script::StringifyJSON(rq, &ais);
 }
