@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2024 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -30,6 +30,8 @@
 #include "ps/Filesystem.h"
 #include "ps/XML/Xeromyces.h"
 
+#include <optional>
+
 #define TS_ASSERT_STREAM(stream, len, buffer) \
 	TS_ASSERT_EQUALS(stream.str().length(), (size_t)len); \
 	TS_ASSERT_SAME_DATA(stream.str().data(), buffer, len)
@@ -40,18 +42,19 @@
 
 class TestComponentManager : public CxxTest::TestSuite
 {
+	std::optional<CXeromycesEngine> xeromycesEngine;
 public:
 	void setUp()
 	{
-		g_VFS = CreateVfs(20 * MiB);
-		TS_ASSERT_OK(g_VFS->Mount(L"", DataDir()/"mods"/"_test.sim", VFS_MOUNT_MUST_EXIST));
-		TS_ASSERT_OK(g_VFS->Mount(L"cache", DataDir()/"_testcache"));
-		CXeromyces::Startup();
+		g_VFS = CreateVfs();
+		TS_ASSERT_OK(g_VFS->Mount(L"", DataDir() / "mods" / "_test.sim" / "", VFS_MOUNT_MUST_EXIST));
+		TS_ASSERT_OK(g_VFS->Mount(L"cache", DataDir() / "_testcache" / "", 0, VFS_MAX_PRIORITY));
+		xeromycesEngine.emplace();
 	}
 
 	void tearDown()
 	{
-		CXeromyces::Terminate();
+		xeromycesEngine.reset();
 		g_VFS.reset();
 		DeleteDirectory(DataDir()/"_testcache");
 	}
@@ -59,14 +62,14 @@ public:
 	void test_Load()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 	}
 
 	void test_LookupCID()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
 		TS_ASSERT_EQUALS(man.LookupCID("Test1A"), (int)CID_Test1A);
@@ -76,7 +79,7 @@ public:
 	void test_AllocateNewEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 
 		TS_ASSERT_EQUALS(man.AllocateNewEntity(), (u32)2);
 		TS_ASSERT_EQUALS(man.AllocateNewEntity(), (u32)3);
@@ -96,10 +99,36 @@ public:
 		TS_ASSERT_EQUALS(man.AllocateNewLocalEntity(), (u32)FIRST_LOCAL_ENTITY);
 	}
 
+	void test_rng()
+	{
+		// Ensure we get the same random number with the same seed
+		double first;
+		{
+			CSimContext context;
+			CComponentManager man(context, *g_ScriptContext);
+			man.SetRNGSeed(123);
+
+			if (!man.m_ScriptInterface.MathRandom(first))
+				TS_FAIL("Couldn't get random number!");
+		}
+
+		double second;
+		{
+			CSimContext context;
+			CComponentManager man(context, *g_ScriptContext);
+			man.SetRNGSeed(123);
+
+			if (!man.m_ScriptInterface.MathRandom(second))
+				TS_FAIL("Couldn't get random number!");
+		}
+
+		TS_ASSERT_EQUALS(first, second);
+	}
+
 	void test_AddComponent_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		CEntityHandle hnd1 = man.AllocateEntityHandle(1);
 
@@ -109,20 +138,20 @@ public:
 		{
 			TestLogger log;
 			TS_ASSERT(! man.AddComponent(hnd1, 12345, noParam));
-			TS_ASSERT_WSTR_CONTAINS(log.GetOutput(), L"ERROR: Invalid component id 12345");
+			TS_ASSERT_STR_CONTAINS(log.GetOutput(), "ERROR: Invalid component id 12345");
 		}
 
 		{
 			TestLogger log;
 			TS_ASSERT(! man.AddComponent(hnd1, CID_Test1B, noParam));
-			TS_ASSERT_WSTR_CONTAINS(log.GetOutput(), L"ERROR: Multiple components for interface ");
+			TS_ASSERT_STR_CONTAINS(log.GetOutput(), "ERROR: Multiple components for interface ");
 		}
 	}
 
 	void test_QueryInterface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
@@ -147,7 +176,7 @@ public:
 	void test_SendMessage()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3, ent4 = 4;
@@ -221,7 +250,7 @@ public:
 	void test_ParamNode()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
@@ -242,7 +271,7 @@ public:
 	void test_script_basic()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test.js"));
 
@@ -286,7 +315,7 @@ public:
 	void test_script_helper_basic()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-helper.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/helpers/test-helper.js"));
@@ -303,7 +332,7 @@ public:
 	void test_script_global_helper()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-global-helper.js"));
 
@@ -319,7 +348,7 @@ public:
 	void test_script_interface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/interfaces/test-interface.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-interface.js"));
@@ -337,24 +366,22 @@ public:
 	void test_script_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 
-		{
-			TestLogger log;
-			TS_ASSERT(man.LoadScript(L"simulation/components/error.js"));
-			// In SpiderMonkey 1.6, JS_ReportError calls the error reporter even if it's inside
-			// a try{} in the script; in recent versions (not sure when it changed) it doesn't
-			// so the error here won't get reported.
-			TS_ASSERT_WSTR_NOT_CONTAINS(log.GetOutput(), L"ERROR: JavaScript error: simulation/components/error.js line 4\nInvalid interface id");
-		}
+		TestLogger log;
+		TS_ASSERT(man.LoadScript(L"simulation/components/error.js"));
+		// The following exception is caught and dropped by the JS script, and should not appear in the logs.
+		TS_ASSERT_STR_NOT_CONTAINS(log.GetOutput(), "ERROR: JavaScript error: simulation/components/error.js line 4\nInvalid interface id");
+		// The following exception is not caught by the JS script.
+		TS_ASSERT_STR_CONTAINS(log.GetOutput(), "ERROR: No script wrapper found for interface id 12345");
 	}
 
 	void test_script_entityID()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-entityid.js"));
@@ -374,7 +401,7 @@ public:
 	void test_script_QueryInterface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-query.js"));
 
@@ -395,7 +422,7 @@ public:
 	void test_script_AddEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-addentity.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/addentity/test-addentity.js"));
@@ -428,7 +455,7 @@ public:
 	void test_script_AddLocalEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-addentity.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/addentity/test-addentity.js"));
@@ -461,7 +488,7 @@ public:
 	void test_script_DestroyEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-destroyentity.js"));
 
@@ -481,7 +508,7 @@ public:
 	void test_script_messages()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-msg.js"));
 
@@ -514,7 +541,7 @@ public:
 	void test_script_template()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-param.js"));
 
@@ -536,7 +563,7 @@ public:
 	void test_script_template_readonly()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-param.js"));
 
@@ -558,7 +585,7 @@ public:
 	void test_script_hotload()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-hotload1.js"));
@@ -591,13 +618,38 @@ public:
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent4, IID_Test1))->GetX(), 200);
 	}
 
+	void test_script_modding()
+	{
+		CSimContext context;
+		CComponentManager man(context, *g_ScriptContext);
+		man.LoadComponentTypes();
+
+		CParamNode testParam;
+		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>100</x>"), PSRETURN_OK);
+
+		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+
+		TS_ASSERT(man.LoadScript(L"simulation/components/test-modding1.js"));
+
+		man.AddComponent(hnd1, man.LookupCID("Modding"), testParam);
+		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 100);
+
+		TS_ASSERT(man.LoadScript(L"simulation/components/test-modding2.js"));
+
+		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 1000);
+		man.AddComponent(hnd2, man.LookupCID("Modding"), testParam);
+		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 1000);
+	}
+
 	void test_serialization()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 
-		entity_id_t ent1 = 1, ent2 = 2, ent3 = FIRST_LOCAL_ENTITY;
+		entity_id_t ent1 = 10, ent2 = 20, ent3 = FIRST_LOCAL_ENTITY;
 		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
@@ -621,13 +673,13 @@ public:
 		TS_ASSERT_STR_EQUALS(debugStream.str(),
 				"rng: \"78606\"\n"
 				"entities:\n"
-				"- id: 1\n"
+				"- id: 10\n"
 				"  Test1A:\n"
 				"    x: 11000\n"
 				"  Test2A:\n"
 				"    x: 21000\n"
 				"\n"
-				"- id: 2\n"
+				"- id: 20\n"
 				"  Test1A:\n"
 				"    x: 1234\n"
 				"\n"
@@ -641,30 +693,31 @@ public:
 		std::string hash;
 		TS_ASSERT(man.ComputeStateHash(hash, false));
 		TS_ASSERT_EQUALS(hash.length(), (size_t)16);
-		TS_ASSERT_SAME_DATA(hash.data(), "\x1c\x45\x2b\x20\x1f\x0c\x00\x93\x60\x78\xe2\x63\xb1\x47\x08\x19", 16);
-		// echo -en "\x05\x00\x00\x0078606\x01\0\0\0\x01\0\0\0\xf8\x2a\0\0\x02\0\0\0\xd2\x04\0\0\x04\0\0\0\x01\0\0\0\x08\x52\0\0" | openssl md5 | perl -pe 's/(..)/\\x$1/g'
-		//           ^^^^^^^^ rng ^^^^^^^^ ^^Test1A^^ ^^^ent1^^ ^^^11000^^^ ^^^ent2^^ ^^^1234^^^ ^^Test2A^^ ^^ent1^^ ^^^21000^^^
+		TS_ASSERT_SAME_DATA(hash.data(), "\x3c\x25\x6e\x22\x58\x23\x09\x58\x38\xca\xb2\x1e\x0b\x8c\xac\xcf", 16);
+		// echo -en "\x05\x00\x00\x0078606\x02\0\0\0\x01\0\0\0\x0a\0\0\0\xf8\x2a\0\0\x14\0\0\0\xd2\x04\0\0\x04\0\0\0\x0a\0\0\0\x08\x52\0\0" | md5sum | perl -pe 's/([0-9a-f]{2})/\\x$1/g'
+		//           ^^^^^^^^ rng ^^^^^^^^ ^^next^^ ^^Test1A^^ ^^^ent1^^ ^^^11000^^^ ^^^ent2^^ ^^^1234^^^ ^^Test2A^^ ^^ent1^^ ^^^21000^^^
 
 		std::stringstream stateStream;
 		TS_ASSERT(man.SerializeState(stateStream));
-		TS_ASSERT_STREAM(stateStream, 69,
+		TS_ASSERT_STREAM(stateStream, 73,
 				"\x05\x00\x00\x00\x37\x38\x36\x30\x36" // RNG
 				"\x02\x00\x00\x00" // next entity ID
+				"\x00\x00\x00\x00" // num system component types
 				"\x02\x00\x00\x00" // num component types
 				"\x06\x00\x00\x00Test1A"
 				"\x02\x00\x00\x00" // num ents
-				"\x01\x00\x00\x00" // ent1
+				"\x0a\x00\x00\x00" // ent1
 				"\xf8\x2a\x00\x00" // 11000
-				"\x02\x00\x00\x00" // ent2
+				"\x14\x00\x00\x00" // ent2
 				"\xd2\x04\x00\x00" // 1234
 				"\x06\x00\x00\x00Test2A"
 				"\x01\x00\x00\x00" // num ents
-				"\x01\x00\x00\x00" // ent1
+				"\x0a\x00\x00\x00" // ent1
 				"\x08\x52\x00\x00" // 21000
 		);
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, *g_ScriptContext);
 		man2.LoadComponentTypes();
 
 		TS_ASSERT(man2.QueryInterface(ent1, IID_Test1) == NULL);
@@ -683,7 +736,8 @@ public:
 	void test_script_serialization()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+
+		CComponentManager man(context, *g_ScriptContext);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
@@ -700,7 +754,12 @@ public:
 
 		man.AddComponent(hnd1, man.LookupCID("TestScript1_values"), testParam);
 		man.AddComponent(hnd2, man.LookupCID("TestScript1_entity"), testParam);
+
+		// TODO: Since the upgrade to SpiderMonkey v24 this test won't be able to correctly represent
+		// non-tree structures because sharp variables were removed (bug 566700).
+		// This also affects the debug serializer and it could make sense to implement correct serialization again.
 		man.AddComponent(hnd3, man.LookupCID("TestScript1_nontree"), testParam);
+
 		man.AddComponent(hnd4, man.LookupCID("TestScript1_custom"), testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 1234);
@@ -713,48 +772,48 @@ public:
 		std::stringstream debugStream;
 		TS_ASSERT(man.DumpDebugState(debugStream, true));
 		TS_ASSERT_STR_EQUALS(debugStream.str(),
-				"rng: \"78606\"\n"
-				"entities:\n"
-				"- id: 1\n"
-				"  TestScript1_values:\n"
-				"    object: {\n"
-				"  \"x\": 1234,\n"
-				"  \"str\": \"this is a string\",\n"
-				"  \"things\": {\n"
-				"    \"a\": 1,\n"
-				"    \"b\": \"2\",\n"
-				"    \"c\": [\n"
-				"      3,\n"
-				"      \"4\",\n"
-				"      [\n"
-				"        5,\n"
-				"        []\n"
-				"      ]\n"
-				"    ]\n"
-				"  }\n"
-				"}\n"
-				"\n"
-				"- id: 2\n"
-				"  TestScript1_entity:\n"
-				"    object: {}\n"
-				"\n"
-				"- id: 3\n"
-				"  TestScript1_nontree:\n"
-				"    object: ({x:#2=[#1=[2], #1#, #2#, {y:#1#}]})\n"
-				"\n"
-				"- id: 4\n"
-				"  TestScript1_custom:\n"
-				"    object: {\n"
-				"  \"c\": 1\n"
-				"}\n"
-				"\n"
+				"rng: \"78606\"\n\
+entities:\n\
+- id: 1\n\
+  TestScript1_values:\n\
+    comp: {\n\
+  \"x\": 1234,\n\
+  \"str\": \"this is a string\",\n\
+  \"things\": {\n\
+    \"a\": 1,\n\
+    \"b\": \"2\",\n\
+    \"c\": [\n\
+      3,\n\
+      \"4\",\n\
+      [\n\
+        5,\n\
+        []\n\
+      ]\n\
+    ]\n\
+  }\n\
+}\n\
+\n\
+- id: 2\n\
+  TestScript1_entity:\n\
+    comp: {}\n\
+\n\
+- id: 3\n\
+  TestScript1_nontree:\n\
+    comp: ({x:[[2], [2], [], {y:[2]}]})\n\
+\n\
+- id: 4\n\
+  TestScript1_custom:\n\
+    comp: {\n\
+  \"c\": 1\n\
+}\n\
+\n"
 		);
 
 		std::stringstream stateStream;
 		TS_ASSERT(man.SerializeState(stateStream));
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, *g_ScriptContext);
 		man2.LoadComponentTypes();
 		TS_ASSERT(man2.LoadScript(L"simulation/components/test-serialize.js"));
 
@@ -771,7 +830,7 @@ public:
 	void test_script_serialization_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
 
@@ -781,15 +840,17 @@ public:
 
 		man.AddComponent(hnd1, man.LookupCID("TestScript1_getter"), noParam);
 
+		TestLogger log;
 		std::stringstream stateStream;
-		TS_ASSERT(man.SerializeState(stateStream));
+		TS_ASSERT_THROWS_PSERROR(man.SerializeState(stateStream), PSERROR_Serialize_ScriptError, "Cannot serialize property getters");
 		// (The script will die if the getter is executed)
 	}
 
 	void test_script_serialization_template()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+
+		CComponentManager man(context, *g_ScriptContext);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
 		man.InitSystemEntity();
@@ -803,7 +864,7 @@ public:
 		TS_ASSERT(man.AddComponent(man.GetSystemEntity(), CID_TemplateManager, noParam));
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager));
 
-		const CParamNode* testParam = tempMan->LoadTemplate(ent2, "template-serialize", -1);
+		const CParamNode* testParam = tempMan->LoadTemplate(ent2, "template-serialize");
 
 		man.AddComponent(hnd2, man.LookupCID("TestScript1_consts"), testParam->GetChild("TestScript1_consts"));
 
@@ -813,11 +874,33 @@ public:
 		TS_ASSERT(man.SerializeState(stateStream));
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, *g_ScriptContext);
 		man2.LoadComponentTypes();
 		TS_ASSERT(man2.LoadScript(L"simulation/components/test-serialize.js"));
 
 		TS_ASSERT(man2.DeserializeState(stateStream));
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man2.QueryInterface(ent2, IID_Test1))->GetX(), 12347);
 	}
+
+	void test_dynamic_subscription()
+	{
+		CSimContext context;
+		CComponentManager man(context, *g_ScriptContext);
+		man.LoadComponentTypes();
+
+		entity_id_t ent1 = 1;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+
+		CParamNode noParam;
+
+		man.AddComponent(hnd1, CID_Test1A, noParam);
+		man.AddComponent(hnd1, CID_Test2A, noParam);
+
+		man.DynamicSubscriptionNonsync(MT_RenderSubmit, man.QueryInterface(ent1, IID_Test1), true);
+		man.DynamicSubscriptionNonsync(MT_RenderSubmit, man.QueryInterface(ent1, IID_Test2), true);
+
+		man.DestroyComponentsSoon(ent1);
+		man.FlushDestroyedComponents();
+	}
+
 };
