@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2021 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -24,17 +24,18 @@
 #include "ScenarioEditor/ScenarioEditor.h"
 #include "ScenarioEditor/Tools/Common/ObjectSettings.h"
 #include "ScenarioEditor/Tools/Common/MiscState.h"
-#include "AtlasScript/ScriptInterface.h"
 #include "VariationControl.h"
 
 #include "GameInterface/Messages.h"
 
 #include "wx/busyinfo.h"
+#include "wx/wxcrt.h"
 
 enum
 {
 	ID_ObjectType = 1,
 	ID_ObjectFilter,
+	ID_ObjectExactFilter,
 	ID_PlayerSelect,
 	ID_SelectObject,
 	ID_ToggleViewer,
@@ -65,7 +66,6 @@ class ObjectBottomBar : public wxPanel
 public:
 	ObjectBottomBar(
 		wxWindow* parent,
-		ScenarioEditor& scenarioEditor,
 		Observable<ObjectSettings>& objectSettings,
 		Observable<AtObj>& mapSettings,
 		ObjectSidebarImpl* p
@@ -73,6 +73,7 @@ public:
 
 	void OnFirstDisplay();
 	void ShowActorViewer(bool show);
+	void OnSelectedObjectsChange(const std::vector<AtlasMessage::ObjectID>& selectedObjects);
 
 private:
 	void OnViewerSetting(wxCommandEvent& evt);
@@ -83,17 +84,15 @@ private:
 	bool m_ViewerMove;
 	bool m_ViewerGround;
 	bool m_ViewerWater;
-	bool m_ViewerShadows;
 	bool m_ViewerPolyCount;
 	bool m_ViewerBoundingBox;
 	bool m_ViewerAxesMarker;
 	int m_ViewerPropPointsMode; // 0 disabled, 1 for point markers, 2 for point markers + axes
 
 	wxPanel* m_ViewerPanel;
+	wxScrolledWindow* m_TemplateNames;
 
 	ObjectSidebarImpl* p;
-	ScenarioEditor& m_ScenarioEditor;
-
 	DECLARE_EVENT_TABLE();
 };
 
@@ -101,8 +100,8 @@ struct ObjectSidebarImpl
 {
 	ObjectSidebarImpl(ScenarioEditor& scenarioEditor) :
 		m_ObjectListBox(NULL), m_ActorViewerActive(false),
-		m_ActorViewerEntity(_T("actor|structures/fndn_1x1.xml")),
-		m_ActorViewerAnimation(_T("idle")), m_ActorViewerSpeed(0.f),
+		m_ActorViewerEntity(L"actor|structures/fndn_1x1.xml"),
+		m_ActorViewerAnimation("idle"), m_ActorViewerSpeed(0.f),
 		m_ObjectSettings(scenarioEditor.GetObjectSettings())
 	{
 	}
@@ -112,14 +111,121 @@ struct ObjectSidebarImpl
 	ObservableScopedConnection m_ToolConn;
 
 	bool m_ActorViewerActive;
-	wxString m_ActorViewerEntity;
-	wxString m_ActorViewerAnimation;
+	std::wstring m_ActorViewerEntity;
+	std::string m_ActorViewerAnimation;
 	float m_ActorViewerSpeed;
 	Observable<ObjectSettings>& m_ObjectSettings;
 
+	struct SearchItem
+	{
+		// Weight of the pattern (higher - better).
+		size_t weight;
+		// Index in the original sequence to save the order.
+		size_t index;
+
+		bool operator<(const SearchItem& other) const
+		{
+			if (weight != other.weight)
+				return weight > other.weight;
+			return index < other.index;
+		}
+	};
+	// Cache the std::vector to prevent frequent reallocations.
+	std::vector<SearchItem> m_SearchItems;
+
+	struct SearchItemFilter
+	{
+		SearchItemFilter(const wxString& filterName)
+		{
+			bool firstCharacterInWord = true;
+			for (const wxUniChar& ch : filterName)
+			{
+				if (wxIsalpha(ch))
+				{
+					if (firstCharacterInWord)
+						m_FilterWords.push_back(wxString());
+					firstCharacterInWord = false;
+					m_FilterWords.back().Append(ch);
+				}
+				else
+					firstCharacterInWord = true;
+			}
+		}
+
+		// Returns positive integer if filter words can present in the name.
+		// Otherwise returns zero.
+		size_t CalculateWeight(const wxString& name)
+		{
+			if (m_FilterWords.empty() || name.IsEmpty())
+				return 0;
+			std::vector<size_t> filterWordWeights(m_FilterWords.size(), 0);
+			for (size_t wordBegin = 0; wordBegin < name.Length();)
+			{
+				if (!wxIsalpha(name[wordBegin]))
+				{
+					++wordBegin;
+					continue;
+				}
+				size_t wordEnd = wordBegin;
+				while (wordEnd < name.Length() && wxIsalpha(name[wordEnd]))
+					++wordEnd;
+				for (size_t i = 0; i < m_FilterWords.size(); ++i)
+				{
+					// TODO: we might have a copy here, maybe we can use
+					// something similar to std::string_view but for wxString.
+					size_t weight = CalculateLCS(
+						m_FilterWords[i], name.Mid(wordBegin, wordEnd - wordBegin));
+					// We use a simple heuristic, if LCS shorter than a half
+					// of the filter word, then it's not good for us.
+					if (weight < m_FilterWords[i].size() / 2)
+						continue;
+					filterWordWeights[i] = std::max(filterWordWeights[i], weight);
+				}
+				wordBegin = wordEnd;
+			}
+			size_t totalWeight = 0;
+			for (size_t weight : filterWordWeights)
+			{
+				// If a filter word was't found at all, then we can't accept
+				// the search item name.
+				if (!weight)
+					return 0;
+				totalWeight += weight;
+			}
+			return totalWeight;
+		}
+
+		// We use longest common subsequence to calculate the weight
+		// between two words: filter and item names.
+		size_t CalculateLCS(const wxString& filterWord, const wxString& word)
+		{
+			if (filterWord.IsEmpty() || word.IsEmpty())
+				return 0;
+			m_LCSGrid.resize(filterWord.Length() * word.Length());
+			for (size_t i = 0; i < filterWord.Length(); ++i)
+				for (size_t j = 0; j < word.Length(); ++j)
+				{
+		#define GRID(x, y) m_LCSGrid[(x) * word.Length() + (y)]
+					GRID(i, j) = wxTolower(filterWord.GetChar(i)) == wxTolower(word.GetChar(j));
+					if (i && j)
+						GRID(i, j) += GRID(i - 1, j - 1);
+					if (i)
+						GRID(i, j) = std::max(GRID(i, j), GRID(i - 1, j));
+					if (j)
+						GRID(i, j) = std::max(GRID(i, j), GRID(i, j - 1));
+		#undef GRID
+				}
+			return m_LCSGrid.back();
+		}
+
+		std::vector<wxString> m_FilterWords;
+		// Cache the std::vector to prevent frequent reallocations.
+		std::vector<size_t> m_LCSGrid;
+	};
+
 	void ActorViewerPostToGame()
 	{
-		POST_MESSAGE(SetActorViewer, ((std::wstring)m_ActorViewerEntity.wc_str(), (std::wstring)m_ActorViewerAnimation.wc_str(), m_ObjectSettings.GetPlayerID(), m_ActorViewerSpeed, false));
+		POST_MESSAGE(SetActorViewer, (m_ActorViewerEntity.c_str(), m_ActorViewerAnimation.c_str(), m_ObjectSettings.GetPlayerID(), m_ActorViewerSpeed, false));
 	}
 };
 
@@ -129,73 +235,85 @@ ObjectSidebar::ObjectSidebar(
 	wxWindow* bottomBarContainer
 )
 	: Sidebar(scenarioEditor, sidebarContainer, bottomBarContainer),
-	  p(new ObjectSidebarImpl(scenarioEditor))
+	m_Impl(new ObjectSidebarImpl(scenarioEditor))
 {
+	wxSizer* scrollSizer = new wxBoxSizer(wxVERTICAL);
+	wxScrolledWindow* scrolledWindow = new wxScrolledWindow(this);
+	scrolledWindow->SetScrollRate(10, 10);
+	scrolledWindow->SetSizer(scrollSizer);
+	m_MainSizer->Add(scrolledWindow, wxSizerFlags().Proportion(1).Expand());
+
 	wxBoxSizer* sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, wxID_ANY, _("Filter")), wxSizerFlags().Align(wxALIGN_CENTER));
+	sizer->Add(new wxStaticText(scrolledWindow, wxID_ANY, _("Filter")), wxSizerFlags().Align(wxALIGN_CENTER));
+	sizer->AddSpacer(2);
 	sizer->Add(
 		Tooltipped(
-			new wxTextCtrl(this, ID_ObjectFilter),
+			new wxTextCtrl(scrolledWindow, ID_ObjectFilter),
 			_("Enter text to filter object list")
 		),
 		wxSizerFlags().Expand().Proportion(1)
 	);
-	m_MainSizer->Add(sizer, wxSizerFlags().Expand());
-	m_MainSizer->AddSpacer(3);
+	scrollSizer->Add(sizer, wxSizerFlags().Expand());
+	scrollSizer->AddSpacer(3);
+	wxCheckBox* exactSearchCheckBox = new wxCheckBox(
+		scrolledWindow, ID_ObjectExactFilter, _("Exact Search"));
+	exactSearchCheckBox->SetValue(true);
+	scrollSizer->Add(Tooltipped(exactSearchCheckBox,
+		_("Provides a search with a strict string equality")));
+	scrollSizer->AddSpacer(3);
 
 	// ------------------------------------------------------------------------------------------
 
 	wxArrayString strings;
 	strings.Add(_("Entities"));
 	strings.Add(_("Actors (all)"));
-	wxChoice* objectType = new wxChoice(this, ID_ObjectType, wxDefaultPosition, wxDefaultSize, strings);
+	wxChoice* objectType = new wxChoice(scrolledWindow, ID_ObjectType, wxDefaultPosition, wxDefaultSize, strings);
 	objectType->SetSelection(0);
-	m_MainSizer->Add(objectType, wxSizerFlags().Expand());
-	m_MainSizer->AddSpacer(3);
-	
-	// ------------------------------------------------------------------------------------------
-
-	p->m_ObjectListBox = new wxListBox(this, ID_SelectObject, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_SINGLE|wxLB_HSCROLL);
-	m_MainSizer->Add(p->m_ObjectListBox, wxSizerFlags().Proportion(1).Expand());
-	m_MainSizer->AddSpacer(3);
+	scrollSizer->Add(objectType, wxSizerFlags().Expand());
+	scrollSizer->AddSpacer(3);
 
 	// ------------------------------------------------------------------------------------------
 
-	m_MainSizer->Add(new wxButton(this, ID_ToggleViewer, _("Switch to Actor Viewer")), wxSizerFlags().Expand());
+	m_Impl->m_ObjectListBox = new wxListBox(scrolledWindow, ID_SelectObject, wxDefaultPosition, wxDefaultSize, 0, nullptr, wxLB_SINGLE|wxLB_HSCROLL);
+	scrollSizer->Add(m_Impl->m_ObjectListBox, wxSizerFlags().Proportion(1).Expand());
+	scrollSizer->AddSpacer(3);
+
+	// ------------------------------------------------------------------------------------------
+
+	scrollSizer->Add(new wxButton(scrolledWindow, ID_ToggleViewer, _("Switch to Actor Viewer")), wxSizerFlags().Expand());
 
 	// ------------------------------------------------------------------------------------------
 
 	m_BottomBar = new ObjectBottomBar(
 		bottomBarContainer,
-		scenarioEditor,
 		scenarioEditor.GetObjectSettings(),
 		scenarioEditor.GetMapSettings(),
-		p
+		m_Impl
 	);
 
-	p->m_ToolConn = scenarioEditor.GetToolManager().GetCurrentTool().RegisterObserver(0, &ObjectSidebar::OnToolChange, this);
+	m_Impl->m_ToolConn = scenarioEditor.GetToolManager().GetCurrentTool().RegisterObserver(0, &ObjectSidebar::OnToolChange, this);
 }
 
 ObjectSidebar::~ObjectSidebar()
 {
-	delete p;
+	delete m_Impl;
 }
 
 void ObjectSidebar::OnToolChange(ITool* tool)
 {
 	if (wxString(tool->GetClassInfo()->GetClassName()) == _T("ActorViewerTool"))
 	{
-		p->m_ActorViewerActive = true;
-		p->ActorViewerPostToGame();
+		m_Impl->m_ActorViewerActive = true;
+		m_Impl->ActorViewerPostToGame();
 		wxDynamicCast(FindWindow(ID_ToggleViewer), wxButton)->SetLabel(_("Return to game view"));
 	}
 	else
 	{
-		p->m_ActorViewerActive = false;
+		m_Impl->m_ActorViewerActive = false;
 		wxDynamicCast(FindWindow(ID_ToggleViewer), wxButton)->SetLabel(_("Switch to Actor Viewer"));
 	}
 
-	static_cast<ObjectBottomBar*>(m_BottomBar)->ShowActorViewer(p->m_ActorViewerActive);
+	static_cast<ObjectBottomBar*>(m_BottomBar)->ShowActorViewer(m_Impl->m_ActorViewerActive);
 }
 
 void ObjectSidebar::OnFirstDisplay()
@@ -207,7 +325,7 @@ void ObjectSidebar::OnFirstDisplay()
 	// Get the list of objects from the game
 	AtlasMessage::qGetObjectsList qry;
 	qry.Post();
-	p->m_Objects = *qry.objects;
+	m_Impl->m_Objects = *qry.objects;
 	// Display first group of objects
 	FilterObjects();
 }
@@ -217,27 +335,52 @@ void ObjectSidebar::FilterObjects()
 	int filterType = wxDynamicCast(FindWindow(ID_ObjectType), wxChoice)->GetSelection();
 	wxString filterName = wxDynamicCast(FindWindow(ID_ObjectFilter), wxTextCtrl)->GetValue();
 
-	p->m_ObjectListBox->Freeze();
-	p->m_ObjectListBox->Clear();
-	for (std::vector<AtlasMessage::sObjectsListItem>::iterator it = p->m_Objects.begin(); it != p->m_Objects.end(); ++it)
+	m_Impl->m_ObjectListBox->Freeze();
+	m_Impl->m_ObjectListBox->Clear();
+	if (wxDynamicCast(FindWindow(ID_ObjectExactFilter), wxCheckBox)->IsChecked() || filterName.IsEmpty())
 	{
-		if (it->type == filterType)
+		for (std::vector<AtlasMessage::sObjectsListItem>::iterator it = m_Impl->m_Objects.begin(); it != m_Impl->m_Objects.end(); ++it)
 		{
+			if (it->type != filterType)
+				continue;
 			wxString id = it->id.c_str();
 			wxString name = it->name.c_str();
 
 			if (name.Lower().Find(filterName.Lower()) != wxNOT_FOUND)
-			{
-				p->m_ObjectListBox->Append(name, new wxStringClientData(id));
-			}
+				m_Impl->m_ObjectListBox->Append(name, new wxStringClientData(id));
 		}
 	}
-	p->m_ObjectListBox->Thaw();
+	else
+	{
+		ObjectSidebarImpl::SearchItemFilter filter(filterName);
+		for (size_t idx = 0; idx < m_Impl->m_Objects.size(); ++idx)
+		{
+			const AtlasMessage::sObjectsListItem& it = m_Impl->m_Objects[idx];
+			if (it.type != filterType)
+				continue;
+			wxString name = it.name.c_str();
+			size_t weight = filter.CalculateWeight(name);
+			// We don't want to show items with zero weight.
+			if (!weight)
+				continue;
+			m_Impl->m_SearchItems.push_back({weight, idx});
+		}
+		std::sort(m_Impl->m_SearchItems.begin(), m_Impl->m_SearchItems.end());
+		for (const ObjectSidebarImpl::SearchItem& item : m_Impl->m_SearchItems)
+		{
+			const AtlasMessage::sObjectsListItem& it = m_Impl->m_Objects[item.index];
+			wxString id = it.id.c_str();
+			wxString name = it.name.c_str();
+			m_Impl->m_ObjectListBox->Append(name, new wxStringClientData(id));
+		}
+		m_Impl->m_SearchItems.clear();
+	}
+	m_Impl->m_ObjectListBox->Thaw();
 }
 
 void ObjectSidebar::OnToggleViewer(wxCommandEvent& WXUNUSED(evt))
 {
-	if (p->m_ActorViewerActive)
+	if (m_Impl->m_ActorViewerActive)
 	{
 		m_ScenarioEditor.GetToolManager().SetCurrentTool(_T(""), NULL);
 	}
@@ -261,11 +404,11 @@ void ObjectSidebar::OnSelectObject(wxCommandEvent& evt)
 
 	// Always update the actor viewer's state even if it's inactive,
 	// so it will be correct when first enabled
-	p->m_ActorViewerEntity = id;
+	m_Impl->m_ActorViewerEntity = id;
 
-	if (p->m_ActorViewerActive)
+	if (m_Impl->m_ActorViewerActive)
 	{
-		p->ActorViewerPostToGame();
+		m_Impl->ActorViewerPostToGame();
 	}
 	else
 	{
@@ -279,11 +422,17 @@ void ObjectSidebar::OnSelectFilter(wxCommandEvent& WXUNUSED(evt))
 	FilterObjects();
 }
 
+void ObjectSidebar::OnToggleExactFilter(wxCommandEvent& WXUNUSED(evt))
+{
+	FilterObjects();
+}
+
 BEGIN_EVENT_TABLE(ObjectSidebar, Sidebar)
 	EVT_CHOICE(ID_ObjectType, ObjectSidebar::OnSelectType)
 	EVT_TEXT(ID_ObjectFilter, ObjectSidebar::OnSelectFilter)
 	EVT_LISTBOX(ID_SelectObject, ObjectSidebar::OnSelectObject)
 	EVT_BUTTON(ID_ToggleViewer, ObjectSidebar::OnToggleViewer)
+	EVT_CHECKBOX(ID_ObjectExactFilter, ObjectSidebar::OnToggleExactFilter)
 END_EVENT_TABLE();
 
 //////////////////////////////////////////////////////////////////////////
@@ -363,18 +512,16 @@ END_EVENT_TABLE();
 
 ObjectBottomBar::ObjectBottomBar(
 	wxWindow* parent,
-	ScenarioEditor& scenarioEditor,
 	Observable<ObjectSettings>& objectSettings,
 	Observable<AtObj>& mapSettings,
 	ObjectSidebarImpl* p
 )
-	: wxPanel(parent, wxID_ANY), p(p), m_ScenarioEditor(scenarioEditor)
+	: wxPanel(parent, wxID_ANY), p(p)
 {
 	m_ViewerWireframe = false;
 	m_ViewerMove = false;
 	m_ViewerGround = true;
 	m_ViewerWater = false;
-	m_ViewerShadows = true;
 	m_ViewerPolyCount = false;
 	m_ViewerBoundingBox = false;
 	m_ViewerAxesMarker = false;
@@ -394,7 +541,10 @@ ObjectBottomBar::ObjectBottomBar(
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerWireframe,   _("Wireframe")),      _("Toggle wireframe / solid rendering")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerMove,        _("Move")),           _("Toggle movement along ground when playing walk/run animations")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerGround,      _("Ground")),         _("Toggle the ground plane")), wxSizerFlags().Expand());
-		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerWater,       _("Water")),          _("Toggle the water plane")), wxSizerFlags().Expand());
+		// TODO: disabled until https://gitea.wildfiregames.com/0ad/0ad/issues/2692 is fixed
+		wxButton* waterButton = new wxButton(m_ViewerPanel, ID_ViewerWater, _("Water"));
+		waterButton->Enable(false);
+		viewerButtonsLeft->Add(Tooltipped(waterButton, _("Toggle the water plane")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerShadows,     _("Shadows")),        _("Toggle shadow rendering")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerPolyCount,   _("Poly count")),     _("Toggle polygon-count statistics - turn off ground and shadows for more useful data")), wxSizerFlags().Expand());
 
@@ -467,10 +617,68 @@ ObjectBottomBar::ObjectBottomBar(
 	mainSizer->Add(playerVariationSizer, wxSizerFlags().Expand());
 
 	// ----------------------------------------------------------------------------------
+	// --- display template name
+	wxSizer* displaySizer = new wxStaticBoxSizer(wxVERTICAL, this, _("Selected entities"));
+	m_TemplateNames = new wxScrolledWindow(this);
+	m_TemplateNames->SetMinSize(wxSize(250, -1));
+	m_TemplateNames->SetScrollRate(0, 5);
+	wxSizer* scrollwindowSizer = new wxBoxSizer(wxVERTICAL);
+	m_TemplateNames->SetSizer(scrollwindowSizer);
+	displaySizer->Add(m_TemplateNames, wxSizerFlags().Proportion(1).Expand());
+	m_TemplateNames->Layout();
+	mainSizer->AddSpacer(3);
+	mainSizer->Add(displaySizer, wxSizerFlags().Proportion(1).Expand());
+
+	g_SelectedObjects.RegisterObserver(0, &ObjectBottomBar::OnSelectedObjectsChange, this);
 
 	SetSizer(mainSizer);
 }
 
+static wxControl* CreateTemplateNameObject(wxWindow* parent, const std::string& templateName, int counterTemplate)
+{
+	wxString idTemplate(wxString::FromUTF8(templateName.c_str()));
+	if (counterTemplate > 1)
+		idTemplate.Append(wxString::Format(wxT(" (%i)"), counterTemplate));
+
+	wxStaticText* templateNameObject = new wxStaticText(parent, wxID_ANY, idTemplate);
+	return templateNameObject;
+}
+
+void ObjectBottomBar::OnSelectedObjectsChange(const std::vector<AtlasMessage::ObjectID>& selectedObjects)
+{
+	Freeze();
+	wxSizer* sizer = m_TemplateNames->GetSizer();
+	sizer->Clear(true);
+
+	AtlasMessage::qGetSelectedObjectsTemplateNames objectTemplatesName(selectedObjects);
+	objectTemplatesName.Post();
+	std::vector<std::string> names = *objectTemplatesName.names;
+
+	int counterTemplate = 0;
+	std::string lastTemplateName = "";
+	for (std::vector<std::string>::const_iterator it = names.begin(); it != names.end(); ++it)
+	{
+		if (lastTemplateName == "")
+			lastTemplateName = (*it);
+
+		if (lastTemplateName == (*it))
+		{
+			++counterTemplate;
+			continue;
+		}
+
+		sizer->Add(CreateTemplateNameObject(m_TemplateNames, lastTemplateName, counterTemplate), wxSizerFlags().Align(wxALIGN_LEFT));
+
+		lastTemplateName = (*it);
+		counterTemplate = 1;
+	}
+	// Add the remaining template
+	sizer->Add(CreateTemplateNameObject(m_TemplateNames, lastTemplateName, counterTemplate), wxSizerFlags().Align(wxALIGN_LEFT));
+
+	Thaw();
+	sizer->FitInside(m_TemplateNames);
+}
+
 void ObjectBottomBar::OnFirstDisplay()
 {
 	// We use messages here because the simulation is not init'd otherwise (causing a crash)
@@ -479,12 +687,11 @@ void ObjectBottomBar::OnFirstDisplay()
 	wxArrayString players;
 	AtlasMessage::qGetPlayerDefaults qryPlayers;
 	qryPlayers.Post();
-	AtObj playerData = AtlasObject::LoadFromJSON(m_ScenarioEditor.GetScriptInterface().GetContext(), *qryPlayers.defaults);
+	AtObj playerData = AtlasObject::LoadFromJSON(*qryPlayers.defaults);
 	AtObj playerDefs = *playerData["PlayerData"];
-	for (AtIter p = playerDefs["item"]; p.defined(); ++p)
-	{
-		players.Add(wxString(p["Name"]));
-	}
+	for (AtIter iterator = playerDefs["item"]; iterator.defined(); ++iterator)
+		players.Add(wxString::FromUTF8(iterator["Name"]));
+
 	wxDynamicCast(FindWindow(ID_PlayerSelect), PlayerComboBox)->SetPlayers(players);
 
 	// Initialise the game with the default settings
@@ -492,7 +699,6 @@ void ObjectBottomBar::OnFirstDisplay()
 	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"walk", m_ViewerMove));
 	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"ground", m_ViewerGround));
 	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"water", m_ViewerWater));
-	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"shadows", m_ViewerShadows));
 	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"stats", m_ViewerPolyCount));
 	POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"bounding_box", m_ViewerBoundingBox));
 	POST_MESSAGE(SetViewParamI, (AtlasMessage::eRenderView::ACTOR, L"prop_points", m_ViewerPropPointsMode));
@@ -525,8 +731,7 @@ void ObjectBottomBar::OnViewerSetting(wxCommandEvent& evt)
 		POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"water", m_ViewerWater));
 		break;
 	case ID_ViewerShadows:
-		m_ViewerShadows = !m_ViewerShadows;
-		POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"shadows", m_ViewerShadows));
+		POST_MESSAGE(SetViewParamB, (AtlasMessage::eRenderView::ACTOR, L"shadows", true));
 		break;
 	case ID_ViewerPolyCount:
 		m_ViewerPolyCount = !m_ViewerPolyCount;
