@@ -1,15 +1,12 @@
 #!/usr/bin/env python
 # coding=UTF-8
 
-from __future__ import absolute_import
-
 import os
 import shutil
 import tempfile
 import unittest
 
 import mozunit
-
 from mozfile import tree
 
 
@@ -19,15 +16,15 @@ class TestTree(unittest.TestCase):
     def test_unicode_paths(self):
         """Test creating tree structure from a Unicode path."""
         try:
-            tmpdir = tempfile.mkdtemp(suffix=u'tmp🍪')
-            os.mkdir(os.path.join(tmpdir, u'dir🍪'))
-            with open(os.path.join(tmpdir, u'file🍪'), 'w') as f:
-                f.write('foo')
+            tmpdir = tempfile.mkdtemp(suffix=u"tmp🍪")
+            os.mkdir(os.path.join(tmpdir, u"dir🍪"))
+            with open(os.path.join(tmpdir, u"file🍪"), "w") as f:
+                f.write("foo")
 
-            self.assertEqual(u'{}\n├file🍪\n└dir🍪'.format(tmpdir), tree(tmpdir))
+            self.assertEqual(u"{}\n├file🍪\n└dir🍪".format(tmpdir), tree(tmpdir))
         finally:
             shutil.rmtree(tmpdir)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     mozunit.main()
