@@ -21,8 +21,6 @@ Example usage:
 
 """
 
-from __future__ import absolute_import
-
 from .moznetwork import get_ip
 
-__all__ = ['get_ip']
+__all__ = ["get_ip"]
